;  ____   __    ___              __       _ 
; |  . \ /  \  /  _/___ |\  /|  /  \  |\ | |
; |  __//  . \(  <_(___)| \/ | /  . \ | \| |
; |_|  (______)\___\    |____|(______)|____|
;
;          Channel F Pac-Man Port 
;               written by 
;                Blackbird
;                   and
;            Fredric Blholtz 
;                2004-2008
; 
; Pac-Man &  of NAMCO LTD., ALL RIGHTS RESERVED.
;
; dasm game.asm -f3 -ogame.bin  to compile

	processor f8

; original: 240 dots, 4 power pills
; F8 version: 185 dots 4 power pills
; fruit original: after 70 and after 170 dots/power pills
; F8 version: 54 dots/power pills and 132 dots/power pills


;[TODO] make directions stored as two, not four bits
;[TODO] ghost looping function to space time/space? (spacetime?)
;[TODO] ghost drawing/clearing function?
;[TODO] erase ghosts first, then redraw them
;[TODO] add in blit bitmasking / tunnel masking


;===========================================================================
; VES Header
;===========================================================================

	include	"ves.h"	

	include	"configuration.inc"

;---------------------------------------------------------------------------
; Register Reference 
;---------------------------------------------------------------------------

; The registers in the F8 are used as both RAM and registers for
; this game. The reference is as follows:

; score must have a 4 in the octal one's digit, to work with score.add
; registers 020 - 027

; hi-score is backed up in RAM, registers can be 
; used for other things inside the game loop

HISCORE_REG			=	020	; #16
ATTACK_COUNTER_REG	=	020	; #16
ATTACK_TIMER_REG		=	021	; #17
GHOSTS_RELEASED_REG	=	022	; #18
GHOST_RELEASE_TIMER_REG =	023	; #19

SCORE_REG			=	024	; #20

; pacman registers (must be in same octet)
PACMAN_X_REG		=	030	; #24
PACMAN_Y_REG		=	031	; #25
PACMAN_DIRECTION_REG	=	032	; #26
PACMAN_ANIMINDEX_REG	=	033	; #27

; ghost registers (must be in same octet)
GHOST_TMP_REG		=	034	; #28
GHOST_TMP_X_REG		=	034	; #28
GHOST_TMP_Y_REG		=	035	; #29
GHOST_TMP_DIR_REG		=	036	; #30		b7 nested, b6 nest.out, b5 first time blue
GHOST_TMP_ANIM_REG	=	037	; #31

; ghost registers (040-057)
GHOST_REG		=	040		; #32
GHOST_0_X_REG	=	040		; #32
GHOST_0_Y_REG	=	041		; #33
GHOST_0_DIR_REG	=	042		; #34
GHOST_0_ANIM_REG	=	043		; #35
GHOST_1_X_REG	=	044		; #36
GHOST_1_Y_REG	=	045		; #37
GHOST_1_DIR_REG	=	046		; #38
GHOST_1_ANIM_REG	=	047		; #39
GHOST_2_X_REG	=	050		; #40
GHOST_2_Y_REG	=	051		; #41
GHOST_2_DIR_REG	=	052		; #42
GHOST_2_ANIM_REG	=	053		; #43
GHOST_3_X_REG	=	054		; #44
GHOST_3_Y_REG	=	055		; #45
GHOST_3_DIR_REG	=	056		; #46
GHOST_3_ANIM_REG	=	057		; #47

; level registers (must be in same octet)
LEVEL_NUMBER_REG	=	060		; #48
GHOST_NUMBER_REG	=	061		; #49
PACMAN_LIVES_REG	=	062		; #50		b7 pac collision, b6 player 1/2, b5 2ply game
GHOSTS_SLOW_REG	=	063		; #51		ghost off loop b7-6, slow ghost b5
POWERPELLETS_REG	=	063		; #51		bits 4-0
GHOST_EATABLE_REG	=	064		; #52		b0 G0, b1 G1, b2 G2, b3 G3
							;	upper four bits "is eaten"
SLOW_TIMER_REG	=	065		; #53
GLOBAL_TIMER_REG	=	066		; #54

BONUS_PRIZE_REG	=	067		; #55		keeps track of when bonus prize is plotted
BONUS_PRIZE_ON	=	070		; #56

;---------------------------------------------------------------------------
; Variables 
;---------------------------------------------------------------------------

; Pac-Man
PACMAN_COLOR		=	green
PACMAN_START_X		=	39
PACMAN_START_Y		=	41
PACMAN_LIVES		=	4

; distance from ghost required for death
DEATH_DISTANCE		=	3

; maze
MAZE_BGCOLOR		=	bkg
MAZE_FGCOLOR		=	blue
PELLET_COLOR		=	red
POWERPELLET_COLOR		=	red
GATE_COLOR			=	red

; sidebar
SIDEBAR_COLOR		=	blue
SCORE_COLOR			=	green
EATSCORE_COLOR		=	red

; ghosts
GHOST_NUMBER		=	4
GHOST_1_COLOR		=	red
GHOST_2_COLOR		=	green
GHOST_3_COLOR		=	blue
GHOST_4_COLOR		=	red

;===========================================================================
; Program Entry
;===========================================================================

;---------------------------------------------------------------------------
; Cartridge Initalization
;---------------------------------------------------------------------------

	org	$800

cartridge.init:
	; initalize the system
	CARTRIDGE_START
	CARTRIDGE_INIT

	; initialize the kstack pointer
	li	62		; stack starts at r62
	lisu	7		; stack pointer is r63
	lisl	7
	lr	S, A
	
;---------------------------------------------------------------------------
; Main Program 
;---------------------------------------------------------------------------

main:

	; setup RAM for intersection check

	pi	program.data.intersection

	; check if highscore is stored in RAM
	; ascii codes for HI is in memory at ram + $600, $601
	; this adress seems to be untouched by other programs
	; except RAM-test (but I'll change it so that it backs it up)

	dci	ram + $600
	lm
	ci	$48
	bnz	main.clear.hiscore
	lm
	ci	$49
	bnz	main.clear.hiscore
	
	br	main.intro

	; no valid highscore header in RAM
	; clear the highscore and store it in RAM
main.clear.hiscore:
	; set header characters for highscore (HI ascii codes)
	dci	ram + $600
	li	$48
	st
	li	$49
	st
	clr
	st
	st
	st
	st
	


main.intro:


	; intro screen, b/w
	li	$21
	lr	3, A
	pi	clrscrn

	
	; plot pacman text logo
	dci	gfx.pacmanlogo.parameters
	pi	blitGraphic

	; plot namco logo
	dci	gfx.namco.parameters
	pi	blitGraphic


	; set middle 25 pixels part to three color, grey bkg
	; change palette (rows [15,40])

	lis	1
	lr	1, A
	; horizontal position (122 = color row (we have offset in plot))
	li	122
	lr	2, A
	; vertical position
	li	27
	lr	3, A
	lr	4, A	
	lr	7, A
	pi	plot
main.grey.field.loop.on:
	ds	7
	lr	A, 7
	lr	3, A
	ci	14
	bz 	main.grey.field.on.end
	pi	plot
	
	; delay	
	lis	5
	lr	5, A
	pi	delay

	lr	A, 4
	inc
	lr	4, A
	lr	3, A
	pi	plot

	; delay	
	lis	5
	lr	5, A
	pi	delay

	br	main.grey.field.loop.on

main.grey.field.on.end:
 


	; ghost x-coordinates stored in r10, r11 as loop register
	li	110
	lr	10, A
	li	%10000100
	lr	11, A

main.ghosts:

main.ghosts.clear:	
	; clear ghosts
	li	bkg
	lr	1, A
	; load x coordinate
	lr	A, 10
	ci	21
	bz	main.ghosts.clear.skip.inc
	lr	A, 11
	ni	%01111000
	ci	8
	bz	main.ghosts.clear.decrease
	lr	A, 11
	ni	%01111000
	bnz	main.ghosts.clear.skip.inc
	lr	A, 10
	inc
	br	main.ghosts.clear.plot
main.ghosts.clear.decrease:
	lr	A, 10
	ai	$ff
	br	main.ghosts.clear.plot
main.ghosts.clear.skip.inc:
	lr	A, 10
main.ghosts.clear.plot:
	lr	2, A
	; load y coordinate
	lr	A, 11
	ni	%00000111
	ci	4
	bz	main.ghosts.clear4	
	ci	3
	bz	main.ghosts.clear3
	ci	2
	bz	main.ghosts.clear2
	ci	1
	bz	main.ghosts.clear1
main.ghosts.clear1:
	lis	$1
	sl	4
	lr	3, A
	br	main.ghosts.clear.y.set
main.ghosts.clear2:
	li	22
	lr	3, A
	br	main.ghosts.clear.y.set
main.ghosts.clear3:
	li	28
	lr	3, A
	br	main.ghosts.clear.y.set
main.ghosts.clear4:
	li	34
	lr	3, A

main.ghosts.clear.y.set:
	clr
	lr	4, A
	; set bitmask
	li	%11111000
	lr	5, A
	; draw the sprite
	pi	sprite.draw


	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	ni	%00000111
	ci	4
	bz	main.ghosts.4	
	ci	3
	bz	main.ghosts.3
	ci	2
	bz	main.ghosts.2
	ci	1
	bz	main.ghosts.1
main.ghosts.1:
	li	red
	lr	1, A
	lis	$1
	sl	4
	lr	3, A
	br	main.ghosts.y.set
main.ghosts.2:
	li	red
	lr	1, A
	li	22
	lr	3, A
	br	main.ghosts.y.set
main.ghosts.3:
	li	blue
	lr	1, A
	li	28
	lr	3, A
	br	main.ghosts.y.set
main.ghosts.4:
	clr				; color green
	lr	1, A
	li	34
	lr	3, A
main.ghosts.y.set:
	; set sprite index
	lr	A, 11
	ni 	%10000000
	bz	main.ghosts.odd
	li	17
	br	main.ghosts.even			
main.ghosts.odd:
	li	18
main.ghosts.even:
	lr	4, A
	; set bitmask
	li	%11111000
	lr	5, A
	; draw the sprite
	pi	sprite.draw

main.readController:
	; see if one of the hand controllers has moved
	clr
	outs	0
	outs	1						; check right hand controller
	ins	1
	com							; un-invert port data
	ni	%11000000					; only keep push/pull
	bnz	main.readController.end
	; check the other controller
	clr
	outs	4						; check left hand controller
	ins	4
	com
	ni	%11000000
	bnz	main.readController.end

	; read buttons
	ins	0
	com
	lr	4, A
	li	128
	lr	5, A
debounce: 
	ds	5
	bnz	debounce
	lr	A, 4
	ni	%00001000			; button #4
	bnz	main.buttons.used.four
	lr	A, 4
	ni	%00000100			; button #3
	bnz	main.buttons.used.three
	lr	A, 4
	ni	%00000010			; button #2
	bnz	main.buttons.used.two
	lr	A, 4
	ni	%00000001			; button #1
	bnz	main.buttons.used.one

	br	main.continue
main.readController.end:
	jmp	main.choose.players


main.buttons.used.four:
	dci	$3FFE
	lm	
	ci	$30
	bnz	main.continue	; do nothing

	jmp	play.sound.wacka.wacka

main.buttons.used.three:
	dci	$3FFE
	lm	
	ci	$30
	bnz	main.continue	; do nothing

	jmp	play.sound.siren


main.buttons.used.two:
	jmp	main.game.twoplayers
main.buttons.used.one:
	jmp	main.game.start

main.continue:
	lr	A, 11
	ai	$ff
	lr	11, A
	ni	%00000111	
	bz	main.ghosts.reloop
	jmp	main.ghosts

main.ghosts.reloop:
	
	lis	4
	lr	5, A
	pi	delay

	lr	A, 11				; skip plotting text when plotted once
	ni	%01111000
	bnz	main.ghosts.keep.moving
	lr	A, 10
	ci	21
	bnz	main.ghosts.keep.moving

	; plot ghosts' names

	dci	gfx.ghostshadow.parameters
	pi	blitGraphic

	dci	gfx.blinky.parameters
	pi	blitGraphic
	dci	gfx.pinky.parameters
	pi	blitGraphic
	dci	gfx.inky.parameters
	pi	blitGraphic
	dci	gfx.clyde.parameters
	pi	blitGraphic

	clr
	lr	Ku, A				; clear timer

	lr	A, 11				; set marker for text plotted
	oi	%01111000
	lr	11, A

main.ghosts.keep.moving:
	
	lr	A, 11
	oi	%00000100	
	lr	11, A
	ni	%10000000
	bz	main.ghosts.anim1
	lr	A, 11
	sl	1
	sr	1
	lr	11, A
	br	main.ghosts.anim2
main.ghosts.anim1
	lr	A, 11
	oi	%10000000
	lr	11, A
main.ghosts.anim2
	lr	A, 10
	ci	21
	bz	main.ghosts.animate
	lr	A, 11				; skip plotting text when plotted once
	ni	%01111000
	bnz	main.ghosts.animate
	ds	10
main.ghosts.animate:

	lr	A, 11
	ni	%01111000
	bz	main.ghosts.stop.or.move
	ci	8
	bz	main.ghosts.move.right
	lr	A, 11
	ni	%10000000
	bnz	main.ghosts.stop.or.move

						; Use Ku as a timer here
	li	20
	lr	5, A
	pi	delay

	lr	A, Ku
	ai	$ff
	lr	Ku, A
	ni	%00000011
	bnz	main.ghosts.stop.or.move

	li	20
	lr	5, A
	pi	delay

	lr	A, 11
	ai	$f8				; subtract 8
	lr	11, A
	

main.ghosts.stop.or.move:
	jmp	main.ghosts

main.ghosts.move.right:
	lr	A, 10
	inc
	lr	10, A
	ci	110
	bz	main.choose.players
	br	main.ghosts.stop.or.move

main.choose.players:
	; clear middle area

	; set sidebar color
	li	MAZE_BGCOLOR
	lr	1, A
	; start row loop
	li	38
	lr	3, A
main.clear.middle.draw.row:
	li	101
	lr	2, A
main.clear.middle.draw.column:
	pi	plot

main.clear.middle.drawCheck:
	; check column
	ds	2
	lr	A, 2
	ci	20						; see if we've reached the left edge
	bnz	main.clear.middle.draw.column

	; check row
	ds	3
	lr	A, 3
	ci	15
	bnz	main.clear.middle.draw.row					; branch if > 0

	dci	gfx.highscore.parameters
	pi	blitGraphic

	; plot current highscore

main.highcore.draw:

	; fetch hi-score from RAM  
	; and store in registers
	dci	ram + $602
	SETISAR	HISCORE_REG
	lm
	lr	I, A
	lm
	lr	I, A
	lm
	lr	I, A
	lm
	lr	S, A

	; load colors
	li	MAZE_BGCOLOR
	lr	1, A
	clr
	lr	2, A

	; load position
	li	54
	lr	10, A
	li	19
	lr	11, A

	; set digit counter
	lis	8
	lr	0, A
	; set ISAR
	SETISAR	HISCORE_REG
	
main.highscore.draw.loop:
	; set DC
	dci	gfx.numbers

	; load size
	lis	3
	lr	5, A
	lis	5
	lr	6, A

	; load position
	lr	A, 10
	lr	3, A
	lr	A, 11
	lr	4, A

	; see if the tile is even or odd
	lis	%1
	ns	0
	bz	main.highscore.draw.evenByte		; if even, branch

main.highscore.draw.oddByte:
	; get lower half-byte
	lis	%1111
	ns	I						; set ISAR to next byte
	br	main.highscore.draw.drawDigit
main.highscore.draw.evenByte:
	; get upper half-byte
	lr	A, S
	sr	4

main.highscore.draw.drawDigit:
	; add tile offset
	sl	1
	adc
	; and draw the number
	pi	blit

	; increase x position
	lis	4
	as	10
	lr	10, A						; each tile is three pixels + one pixel space

	; decrease the counter
	ds	0
	bz	main.highscore.draw.end					; reached the end, break
	br	main.highscore.draw.loop

main.highscore.draw.end:

	clr	
	lr	11, A
	lr	Ku, A

main.players.choice:

	; plot 1 player 2 player text - flash


	; patch
	; first row of graphics in blitGraphics
	; got messed up, this code plots one
	; dot outside visible screen

	li	110
	lr	2, A
	pi	plot


	dci	gfx.1player.parameters
	pi	blitGraphic

	dci	gfx.2players.parameters
	pi	blitGraphic

	; delay
	li	25
	lr	5, A
	pi	delay
						; Use Ku as a timer here
	lr	A, Ku
	ai	$fc
	lr	Ku, A				; restart intro if timer runs out
	bnz	main.players.choice.erase.text

main.intro.startover:
;	jmp	main
	jmp	0


main.players.choice.erase.text:

	; erase one of the player texts

	lr	A, 11
	ni	%00000001
	bnz	main.players.choice2

	; set color
	li	MAZE_BGCOLOR
	lr	1, A
	; start row loop
	li	33
	lr	3, A
main.players.choice1.draw.row:
	li	40
	lr	2, A
main.players.choice1.draw.column:
	pi	plot

main.players.choice1.drawCheck:
	; check column
	ds	2
	lr	A, 2
	ci	11						; see if we've reached the left edge
	bnz	main.players.choice1.draw.column

	; check row
	ds	3
	lr	A, 3
	ci	27
	bnz	main.players.choice1.draw.row					; branch if > 0

	br	main.players.choice.readController

main.players.choice2:

	; set color
	li	MAZE_BGCOLOR
	lr	1, A
	; start row loop
	li	33
	lr	3, A
main.players.choice2.draw.row:
	li	89
	lr	2, A
main.players.choice2.draw.column:
	pi	plot

main.players.choice2.drawCheck:
	; check column
	ds	2
	lr	A, 2
	ci	55						; see if we've reached the left edge
	bnz	main.players.choice2.draw.column

	; check row
	ds	3
	lr	A, 3
	ci	27
	bnz	main.players.choice2.draw.row					; branch if > 0


main.players.choice.readController:
	; see if one of the hand controllers has moved
	clr
	outs	0
	outs	1
	ins	1
	com							; un-invert port data
	ni	%11000011					; only keep push/pull
	lr	10, A
	bnz	main.players.choice.readController.end
	; check the other controller
	clr
	outs	4
	ins	4
	com
	ni	%11000011
	lr	10, A
	bnz	main.players.choice.readController.end

	; read buttons
	ins	0
	com
	lr	4, A
	li	128
	lr	5, A
debounce2: 
	ds	5
	bnz	debounce2
	lr	A, 4
	ni	%00000010			; button #2
	bnz	main.game.twoplayers
	lr	A, 4
	ni	%00000001			; button #1
	bnz	main.game.start

	br	main.players.choice.continue
main.players.choice.readController.end:
	ni	%11000000					; controller pulled/pushed -> start game
	bnz	main.game.start

	clr
	lr	Ku, A						; restart timer after movement

	lr	A, 10						; we have either right or left movement
	ni	%00000010					; check right
	bz	main.players.shift2			; if it's 0 change to right
	clr
	lr	11, A
	; set two-player mode off
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%11011111
	lr	S, A
	br	main.players.choice.continue
main.players.shift2:
	lis	1
	lr	11, A
	; set two-player mode
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	oi	%00100000
	lr	S, A

main.players.choice.continue:

	; delay
	lis	$f
	lr	5, A
	pi	delay

	jmp	main.players.choice

main.game.twoplayers:
	; set two-player mode
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	oi	%00100000
	lr	S, A

main.game.start:
	; display a three color, grey background

	pi	clearscreen

main.startLevel:
	; load the level number
	SETISAR	LEVEL_NUMBER_REG
	lis	1						; we're on level 1
	lr	S, A

	; clear the score
	SETISAR	SCORE_REG
	clr
	lr	I, A
	lr	I, A
	lr	I, A
	lr	S, A

	; load pacman's number of lives
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	oi	PACMAN_LIVES
	lr	S, A

	; setup the pellet RAM
	pi	pellets.init


	; set all Powerpellets on
	SETISAR	POWERPELLETS_REG
	lis	%00001111
	lr	S, A

	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%00100000				; check if 2 player game is selected
	bz	level.init
	; player 2 init backup
	pi	backup.players

;===========================================================================
; level loop
;===========================================================================

;---------------------------------------------------------------------------
; init 
;---------------------------------------------------------------------------

level.init:

	; erase Pac-man

	SETISAR	PACMAN_X_REG
	lr	A, I
	lr	2, A
	lr	A, S
	lr	3, A
	; load color
	li	MAZE_BGCOLOR
	lr	1, A
	; clear index
	clr
	lr	4, A
	; draw the sprite
	pi	sprite.draw

	; setup the pellet RAM
	pi	pellets.init

	; clear bonus_prize_reg
	SETISAR	BONUS_PRIZE_REG
	clr
	lr	S, A
	; clear attack register
	SETISAR	ATTACK_COUNTER_REG
	lr	S, A	

	; set all Powerpellets on
	SETISAR	POWERPELLETS_REG
	lis	%00001111
	lr	S, A

	; draw sidebar
	pi	sidebar.draw
	pi	hiscore.draw
	pi	score.draw

;------------------------------------------------------------------------

level.init.drawScreen:

	; draw the map, set map bg color
	; color 1
	li	MAZE_BGCOLOR
	lr	1, A						; r1
	pi	map.draw
	; pellets
	pi	pellets.draw

	; clear prize

	; set sidebar color
	li	SIDEBAR_COLOR
	lr	1, A

	; start row loop
	li	51
	lr	3, A
level.init.drawScreen.draw.column:
	li	97
	lr	2, A
level.init.drawScreen.draw.plot:
	pi	plot

level.init.drawScreen.drawCheck:
	; check column
	ds	2
	li	76						; see if we've reached the left edge
	xs	2
	bnz	level.init.drawScreen.draw.plot

	; check row
	ds	3
	li	34
	xs	3
	bnz	level.init.drawScreen.draw.column	



	SETISAR	GLOBAL_TIMER_REG		; power pellets will start in on position
	ds	S

	; power pellets
	pi	powerPellets.draw

	; player scores
	pi	score.draw


	; store current level in temp reg to plot prize
	SETISAR	LEVEL_NUMBER_REG
	lr	A, S
	lisl	1						; backup level reg in ghost_number_reg
	lr	S, A
	ni	%00001111					; mask off levels > 15 (they are all key)
	lisl	6						; store masked number in global_timer_reg
	lr	S, A
	lisl	0
	clr							; start counting at 0 
	lr	S, A						; 0 is stored in level_number_reg


level.init.drawScreen.prize:
	SETISAR	LEVEL_NUMBER_REG
	lr	A, S
	inc							; increase level_number_reg
	lr	S, A						; store back in level_number_reg
	lisl	6						; compare with masked version
	xs	S						; count and re-call prize.draw until we hit current
	bz	level.init.drawScreen.prize.end	

	; draw prize in sideborder
	pi	prize.draw	 	

	br	level.init.drawScreen.prize

level.init.drawScreen.prize.end:

	; restore level register
	SETISAR	GHOST_NUMBER_REG
	lr	A, S
	lisl	0
	lr	S, A

	lisl	1	; clear ghost_number_reg
	clr
	lr	S, A
	lisl	6	; set global_timer_reg
	clr
	com
	lr	S, A	; set to 255
	


	; draw "PLAYER 1"
	dci	gfx.player1.parameters
	pi	blitGraphic
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%01000000
	bnz	level.init.drawScreen.player2
	br	level.init.intro
level.init.drawScreen.player2:
	; draw "PLAYER 2"
	dci	gfx.player2.parameters
	pi	blitGraphic


; -----------------------------------------------------------------------
	; jump to here after death
level.init.intro:

	; redraw gate

	; draw the exit gate for the ghost
	li	GATE_COLOR
	lr	1,A
	li	39
	lr	2,A
	li	22
	lr	3,A
	li	19
	lr	4,A
	li	%11111000
	lr	5,A
	pi	sprite.draw	


	; draw the number of lives
	pi	pacman.drawLives

	; restore global timer if there's one backed up 
	SETISAR	ATTACK_TIMER_REG
	lr	A, S
	bz	level.init.intro.skip.timer
	SETISAR	GLOBAL_TIMER_REG
	lr	S, A
	SETISAR	ATTACK_TIMER_REG
	clr
	lr	S, A

level.init.intro.skip.timer:
	SETISAR	GLOBAL_TIMER_REG		; power pellets will start in on position
	lr	A, S
	oi	%00000001	
	lr	S, A

	; power pellets
	pi	powerPellets.draw

	; set bonus prize off 
	SETISAR	BONUS_PRIZE_ON
	clr
	lr	S, A
	SETISAR	GHOSTS_RELEASED_REG
	lr	I, A
	lr	S, A
	; clear prize
	pi	prize.draw

	; draw "READY!"
	dci	gfx.ready.parameters
	pi	blitGraphic	

	; skip music if we're not starting a new game (the score would be 0)
	SETISAR	SCORE_REG
	clr
	xs	I
	bnz	level.init.setupPacman
	xs	I
	bnz	level.init.setupPacman
	xs	I
	bnz	level.init.setupPacman
	xs	S
	bnz	level.init.setupPacman

    IF switch_music
level.init.intro.playSong.0:
	; play the opening song
	dci	sfx.intro.0
	pi	playSong
    ENDIF

	; subtract pacman's current life
	SETISAR	PACMAN_LIVES_REG
	ds	S
	; draw them on-screen
	pi	pacman.drawLives

	IF	switch_music
	br	level.init.setupPacman.nopause
   	ENDIF

level.init.setupPacman:

	; pause
	li	$ff
	lr	5, A
	pi	delay
	li	$ff
	lr	5, A
	pi	delay
	li	$ff
	lr	5, A
	pi	delay


level.init.setupPacman.nopause:
	; load coordinates
	SETISAR	PACMAN_X_REG
	li	PACMAN_START_X
	lr	I, A						; load the pac-man x coordinate
	li	PACMAN_START_Y
	lr	I, A						; load the pac-man y coordinate
	lis	%0010
	lr	I, A						; set the direction, left as the original
	lis	2
	lr	S, A						; load the animation index


level.init.setupGhosts:
	; setup the ghosts
	SETISAR	GHOST_REG
	

	; Blinky / Shadow (red)
level.init.setupGhosts.0:
	; x position
	li	39
	lr	I, A
	; y position
	li	17
	lr	I, A
	; direction
	lis	%0001
	lr	I, A
	; animation index
	clr
	lr	I, A

	; Clyde / Pokey (yellow)
level.init.setupGhosts.1:
	; x position
	li	48
	lr	I, A
	; y position
	li	23		; + 6
	lr	I, A
	; direction
	li	%11000001		; start locked nested
	lr	I, A
	; animation index
	clr
	lr	S, A

	; Bashful / Inky (cyan)
	SETISAR	[GHOST_REG + 010]
level.init.setupGhosts.2:
	; x position
	li	30
	lr	I, A
	; y position
	li	23		; + 6
	lr	I, A
	; direction
	li	%11000001		; start locked nested
	lr	I, A
	; animation index
	clr
	lr	I, A

	; Speedy / Pinky (pink)
level.init.setupGhosts.3:
	; x position
	li	39
	lr	I, A
	; y position
	li	23		; + 6
	lr	I, A
	; direction
	li	%11000001		; start locked nested
	lr	I, A
	; animation index
	clr
	lr	S, A

	; clear player-text and ready text
level.init.text.erase:
 	; load color
	li	MAZE_BGCOLOR
	lr	1, A
	; start row loop
	li	33
	lr	3, A
level.init.text.erase.row:
	li	51
	lr	2, A
level.init.text.erase.column:
	pi	plot

level.init.text.erase.checkLoop:
	; check column
	ds	2
	lr	A, 2
	ci	28						; see if we've reached the left edge
	bnz	level.init.text.erase.column
	; check row
	ds	3
	lr	A, 3
	ci	28
	bnz	level.init.text.erase.row			; branch if > 0


	; the following should be just before start, after start melody

level.init.text2.erase:
 	; load color
	li	MAZE_BGCOLOR
	lr	1, A
	; start row loop
	li	21
	lr	3, A
level.init.text2.erase.row:
	li	56
	lr	2, A
level.init.text2.erase.column:
	pi	plot

level.init.text2.erase.checkLoop:
	; check column
	ds	2
	lr	A, 2
	ci	26				; see if we've reached the left edge
	bnz	level.init.text2.erase.column
	; check row
	ds	3
	lr	A, 3
	ci	16
	bnz	level.init.text2.erase.row			; branch if > 0

level.init.drawPacman:
	; draw Pac-Man
	pi	pacman.draw

	; load first ghost
	SETISAR GHOST_NUMBER_REG
	clr
	lr	S, A
level.init.drawGhosts:
	; load this ghost's coordinates
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	0, A

	; load the ghost color
	dci	gfx.ghost.colors
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	adc
	lm
	lr	1, A
	
	; load the saved ISAR
	lr	A, 0
	lr	IS, A
	; load x coordinate
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	; set sprite index
	li	17								; set all ghosts to first anim.
	lr	4, A
	; set bitmask
	li	%11111000
	lr	5, A
	; draw the sprite
	pi	sprite.draw

level.init.drawGhosts.checkLoop:
	; check if we've reached the last ghost
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	inc
	ci	GHOST_NUMBER
	bz	level.init.drawGhosts.end
	; update next ghost
	lr	S, A
	br	level.init.drawGhosts
level.init.drawGhosts.end:


	; no ghosts are eatable	nor eaten
	SETISAR	GHOST_EATABLE_REG
	clr
	lr	S, A

	SETISAR	SCORE_REG
	clr
	; check that we're starting a new game (the score would be 0)
	xs	I
	bnz	level.init.delay
	xs	I
	bnz	level.init.delay
	xs	I
	bnz	level.init.delay
	xs	S
	bnz	level.init.delay


    IF switch_music
level.init.intro.playSong.1:
	; play the opening song
	dci	sfx.intro.1
	pi	playSong
	br	level.loop
    ENDIF



level.init.delay:
	; delay for a short bit
	lis	$f
	lr	5, A
	pi	delay

;---------------------------------------------------------------------------
; loop
;---------------------------------------------------------------------------

level.loop:


	; Either draw the score or powerpellets

	SETISAR	GLOBAL_TIMER_REG
	lr	A, S
	ni	%00000001
	bnz	level.loop.powerpellets
	


	; release ghosts

	SETISAR	GHOSTS_RELEASED_REG
	clr
	as	S
	bnz	level.loop.continue

	SETISAR 	LEVEL_NUMBER_REG
	lr	A, I
	ni	%11111100
	bnz	level.loop.timer.start.all.ghosts	
	; choose ghost depending on timer
	SETISAR	GHOST_RELEASE_TIMER_REG
	ds	S
	lr	A, S
	ci	$F2
	bz	level.loop.timer.start.pinky
	ci	$C0
	bz	level.loop.timer.start.inky
	ci	$95
	bz	level.loop.timer.start.pokey
	br	level.loop.continue


level.loop.timer.start.pinky:
	; 'o'56 = Pinky  [56]
	lisu	5
	lisl	6
	br	level.loop.timer.ghost.set

level.loop.timer.start.inky:
	; 'o'52 = Inky  	[52]
	lisu	5
	lisl	2
	br	level.loop.timer.ghost.set

level.loop.timer.start.pokey:
	SETISAR	GHOSTS_RELEASED_REG
	ds	S
	SETISAR	[GHOST_REG + 006]	
	; +6     = Pokey  [46]
level.loop.timer.ghost.set:
	lr	A, S
	ni	%01111111				; release
	lr	S, A
	br	level.loop.continue


	; replaced ni %01111111 with sl sr from b29b

level.loop.timer.start.all.ghosts:
	SETISAR	[GHOST_REG + 6]
	lr	A, S
	sl	1				; release pokey
	sr	1
	lr	S, A
	lisu	5
	lr	A, S
	sl	1				; release pinky
	sr	1
	lr	S, A
	lisl	2
	lr	A, S
	sl	1				; release inky
	sr	1
	lr	S, A


level.loop.continue:
	; draw the player score
	pi	score.draw
	br	level.loop.timer.decrease

level.loop.powerpellets:
	pi	powerPellets.draw

level.loop.timer.decrease:
	; decrease global timer
	SETISAR	GLOBAL_TIMER_REG
	ds	S

level.loop.timer.decrease.prize:
	; decrease bonus_prize timer if prize is active
	SETISAR	BONUS_PRIZE_ON
	clr
	as	S
	bz	level.loop.timer.pellets
	ds	S
	lr	A, S
	ci	128
	bnz	level.loop.timer.pellets				; jump if timer isn't 128

	; clear bonus_prize_on and erase prize 
	clr
	lr	S, A							
	pi	prize.draw

level.loop.timer.pellets:
	; decrease timer if ghosts are eatable

	SETISAR	POWERPELLETS_REG
	lr	A, I							; check if slow bit is set
	ni	%00100000
	bz	level.loop.pacman.readController		; branch if not
	lr	A, I							; set correct ISAR for timer
;	li	$fd
	lis	%0010
	com
	as	S							; decrease SLOW_TIMER_REG	
	lr	S, A
	bz	level.loop.resetSlow				; timer zero stops slow-mode
	br	level.loop.pacman.readController

level.loop.resetSlow:
	SETISAR	POWERPELLETS_REG
	lr	A, S
	ni	%00011111
	lr	I, A							; also change ISAR to ghost_eatable_reg
	lr	A, S
	ni	%11110000						; no ghosts eatable
	lr	S, A

	; restore global timer 
	SETISAR	ATTACK_TIMER_REG
	lr	A, S
	SETISAR	GLOBAL_TIMER_REG
	lr	S, A
	SETISAR	ATTACK_TIMER_REG
	clr
	lr	S, A

;---------------------------------------------------------------------------
; pacman
;---------------------------------------------------------------------------

	; spool for controller input
level.loop.pacman.readController:

	; When two player game only read one controller

	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%00100000				; check if 2 player game is selected
	bnz	level.loop.pacman.readController.two.players

	; see if either hand controllers has moved
	clr
	outs	0
	outs	1						; check right controller
	ins	1
	com							; un-invert port data


	; debug

	IF	switch_debug_v

	nop

	ELSE

	sl	4						; mask off twists, since they're unused
	sr	4

	ENDIF


	bnz	level.loop.pacman.readController.end
	; check the other controller
	clr
	outs	4
	ins	4						; check left controller
	com


	; debug


	IF	switch_debug_v

	nop

	ELSE

	sl	4						; mask off twists, since they're unused
	sr	4

	ENDIF


level.loop.pacman.readController.end:
	; save the input


	IF	switch_debug_v
	
	lr	6, A

	ni	%11110000
	bz	continue.as.usual

	jmp	level.cleared


continue.as.usual:
	lr	A, 6

	sl	4
	sr	4

	ENDIF
	
	lr	6, A


	br	level.loop.pacman.handleInput


level.loop.pacman.readController.two.players:
	; skip right controller if player 2 is active
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%01000000
	bnz	level.loop.pacman.readController.two.players.second
	
	clr
	outs	0
	outs	1						; check right controller
	ins	1
	com							; un-invert port data
	sl	4						; mask off twists, since they're unused
	sr	4
	br	level.loop.pacman.readController.two.players.end
level.loop.pacman.readController.two.players.second:
	clr
	outs	0
	outs	4
	ins	4						; check left controller
	com
	sl	4
	sr	4
level.loop.pacman.readController.two.players.end:
	; save the input
	lr	6, A



level.loop.pacman.handleInput:
	; check to see if we're on an intersection
	pi	pacman.checkIntersection
	; branch if false
	ci	1
	bnz	level.loop.pacman.handleInput.noIntersection

level.loop.pacman.handleInput.inIntersection:
	; ensure only one direction is selected
	lr	A, 6
	ni	%0111
	bz	level.loop.pacman.handleInput.inIntersection.end
	lr	6, A
	ni	%0011
	bz	level.loop.pacman.handleInput.inIntersection.end
	lr	6, A
	ni	%0001
	bz	level.loop.pacman.handleInput.inIntersection.end
	lr	6, A
level.loop.pacman.handleInput.inIntersection.end:
	; load the boundaries
	lr	A, 0
	; compare and set direction
	br	level.loop.pacman.handleInput.set

level.loop.pacman.handleInput.noIntersection:
	; if you're not in an intersection,
	; you can only go the opposite way
	; you're going right now

	; set ISAR for direction
	SETISAR	PACMAN_DIRECTION_REG
	; up
	lis	%1000
	ns	S
	bnz	level.loop.pacman.handleInput.up
	; down
	lis	%0100
	ns	S
	bnz	level.loop.pacman.handleInput.down
	; left
	lis	%0010
	ns	S
	bnz	level.loop.pacman.handleInput.left
	; right
	lis	%0001
	ns	S
	bnz	level.loop.pacman.handleInput.right
	; none
	clr
	br	level.loop.pacman.handleInput.set

level.loop.pacman.handleInput.up:
	lis	%0100
	br	level.loop.pacman.handleInput.set
level.loop.pacman.handleInput.down:
	lis	%1000
	br	level.loop.pacman.handleInput.set
level.loop.pacman.handleInput.left:
	lis	%0001
	br	level.loop.pacman.handleInput.set
level.loop.pacman.handleInput.right:
	lis	%0010
level.loop.pacman.handleInput.set:
	; see if the direction is allowed, otherwise end routine
	ns	6
	bz	level.loop.pacman.handleInput.end

	; set the direction register
	SETISAR	PACMAN_DIRECTION_REG
	lr	S, A

level.loop.pacman.handleInput.end:


	; Moved Pac-man clear from v b29b from here to next mark.

	; clear pac-man
	pi	pacman.clear



level.loop.pacman.checkDirection:
	; check that pacman's current direction is still valid
	; allowed directions returned in r0
	pi	pacman.checkIntersection
	ci	1
	bnz	level.loop.pacman.checkDirection.end			; not in an intersection

	; clear direction if necessary
	SETISAR	PACMAN_DIRECTION_REG
	lr	A, S
	ns	0
	bnz	level.loop.pacman.checkDirection.end
	br	level.loop.pacman.draw				; can't go anywhere, so just skip updating pacman	
level.loop.pacman.checkDirection.end:

level.loop.pacman.update:
	SETISAR	PACMAN_ANIMINDEX_REG
	lr	A, S						; load the sprite index
	inc							; increase it
	ni	%00000011					; clear if on frame 4
	lr	S, A						; save the new index

level.loop.pacman.move:
	; now we move pac-man based on the direction
	; which is set in r037


	; moved pacman.clear from here - made a graphics glitch


	SETISAR	PACMAN_DIRECTION_REG
	lr	A,S						; check which bit is 1 by shifting right
	; right (%0001)
	sr	1
	bz	level.loop.pacman.move.right
	; left (%0010)
	sr	1
	bz	level.loop.pacman.move.left
	; down (%0100)
	sr	1
	bz	level.loop.pacman.move.down
	; up (%1000)
	sr	1
	bz	level.loop.pacman.move.up
	; none (%0000)
	br	level.loop.pacman.move.end

level.loop.pacman.move.up:
	; subtract y by 1
;	SETISAR	PACMAN_Y_REG
	lisl	1
	ds	S						; subtract 1
	br	level.loop.pacman.move.end
level.loop.pacman.move.right:
	; add x by 1
;	SETISAR	PACMAN_X_REG
	lisl	0
	lr	A, S
	inc							; add 1
	lr	S, A
	br	level.loop.pacman.move.end
level.loop.pacman.move.down:
	; add y by 1
;	SETISAR	PACMAN_Y_REG
	lisl	1
	lr	A, S
	inc							; add 1
	lr	S, A
	br	level.loop.pacman.move.end
level.loop.pacman.move.left:
	; subtract x by 1
;	SETISAR	PACMAN_X_REG
	lisl	0
	ds	S						; subtract 1
	br	level.loop.pacman.move.end
level.loop.pacman.move.end:

	; check if Pac-Man has moved through the tunnel
level.loop.pacman.checkTunnel:
	; check Y position
;	SETISAR PACMAN_Y_REG
	lisl	1
	lr	A, D
	ci	$17
	bnz	level.loop.pacman.checkTunnel.end
	; check X position
	lr	A, S
	ci	$48
	bz	level.loop.pacman.checkTunnel.warpRight		; Pac-Man has passed through 
	ci	$6								; left side
	bnz	level.loop.pacman.checkTunnel.end

level.loop.pacman.checkTunnel.warpLeft:
	; set new position on right side
	li	$47
	lr	S, A
	br	level.loop.pacman.checkTunnel.end	
level.loop.pacman.checkTunnel.warpRight:
	; set new position on left side
	lis	$7
	lr	S, A
level.loop.pacman.checkTunnel.end:

level.loop.pacman.draw:
	; draw Pac-Man
	pi	pacman.draw
	
;---------------------------------------------------------------------------
; Ghosts 
;---------------------------------------------------------------------------


	SETISAR	POWERPELLETS_REG
	lr	A, S
	ni	%00100000				; check slow ghosts bit
	bz	level.loop.1
	lis	$4					; counter to skip sound
	sl	4
	as	S
	lr	S, A
	ni	%11000000				; how often to play active sound 
	bnz	level.loop.1

	IF	switch_music
	; sound when powerpellet is active
	dci	sfx.powerpellet
	pi	playSong
	ENDIF

level.loop.1:
	; ghost loop
	; loops through each of the four ghosts 
	; updating position and direction
	SETISAR GHOST_NUMBER_REG
	clr
	lr	S, A					; start on Ghost 0
level.loop.ghosts:
	
	; move registers to temp location
	; get ISAR location for ghost
	SETISAR GHOST_NUMBER_REG
	; multiply by 4 bytes per ghost
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	IS, A
	; load registers
	lr	A, I
	lr	0, A
	lr	A, I
	lr	1, A
	clr
	as	I

	; bug repair, check if direction is 00 - correct to 80
	
	bz	level.loop.ghosts.repair	
level.loop.ghosts.repair.done:


	lr	2, A
	lr	A, S
	lr	3, A
	; store at temp ghost position
	SETISAR	GHOST_TMP_REG
	lr	A, 0
	lr	I, A
	lr	A, 1
	lr	I, A
	lr	A, 2
	lr	I, A
	lr	A, 3
	lr	S, A
	br	level.loop.ghosts.repair.skip

level.loop.ghosts.repair:
	li	$80
	br	level.loop.ghosts.repair.done
level.loop.ghosts.repair.skip:
	


	; any ghosts out of nest?
	SETISAR	GHOST_TMP_DIR_REG
	lr	A, S
	sr	4
	ci	8					; if 8, ghost is nested
	bz	level.loop.ghosts.nesting
	ci	4					; if 4, ghost is nested but on it's way out
	bz	level.loop.ghosts.nesting
	br	level.loop.ghosts.eaten.check

level.loop.ghosts.nesting:
	pi	ghost.nest
	jmp	level.loop.ghosts.draw	


	; jump to dead-ghost-movement if any ghost is eaten
level.loop.ghosts.eaten.check:


	SETISAR	GHOST_EATABLE_REG
	lr	A, S
	sr	4							; -> 0000uuuu
	bz	level.loop.ghosts.start.update		; skip this if no ghost is eaten

	; check if current ghost is already eaten
	SETISAR	GHOST_NUMBER_REG
	lr	A, S							; load ghost number in A
	dci	eatable.mask.data					; set DC
	adc								; add ghost number as offset
	lm								; read ROM
	sl	4							; <- LLLL0000
	lisl	4							; Set register eatable
	ns	S							; AND register
	bz	level.loop.ghosts.start.update			; this ghost is not eaten
	jmp	level.loop.ghosts.return.nest			; current ghost eaten

	; skip updating ghost position every 4th time to 
	; slow ghosts down when "slow ghosts" are activated

level.loop.ghosts.start.update:



	; no ghost is eaten
	SETISAR	POWERPELLETS_REG
	lr	A, S
	ni	%00100000						; check slow ghosts bit
	bnz	level.loop.ghosts.skip.check

	; jump to retreat mode if counter is correct
	SETISAR	ATTACK_COUNTER_REG

	IF	switch_no_attack

	br	level.loop.ghosts.start.update.quadrant.move

	ENDIF

	IF	switch_retreat
	clr
	as	S
	bz	level.loop.ghosts.start.update.quadrant.move
	ci	2
	bz	level.loop.ghosts.start.update.quadrant.move
	ci	4
	bz	level.loop.ghosts.start.update.quadrant.move
	ci	6
	bz	level.loop.ghosts.start.update.quadrant.move
	ci	7
	bt	3,level.loop.ghosts.start.update.timercheck

	ENDIF

	lis	7
	lr	S, A
level.loop.ghosts.start.update.timercheck:
	SETISAR	GLOBAL_TIMER_REG
	lr	A, S
	sr	1
	ci	$08
	bm	level.loop.ghosts.checkDirection
	SETISAR	ATTACK_COUNTER_REG			; increase attack counter
	lis	1
	as	S
	lr	S, A
	; clear global_timer
	SETISAR	GLOBAL_TIMER_REG
	clr
	lr	S, A							; clear global timer and update
	ds	S
	br	level.loop.ghosts.checkDirection

level.loop.ghosts.start.update.quadrant.move:
	jmp	level.loop.ghosts.quadrant.move

level.loop.ghosts.skip.check:
	lr	A, S							; check "skip counter"
	ni	%11000000
	bz	level.loop.ghosts.check				; 
	br	level.loop.ghost.flee.checkDirection

level.loop.ghosts.check:

	; check if current ghost is eatable
	SETISAR	GHOST_NUMBER_REG
	lr	A, S
	dci	eatable.mask.data					; set correct data table
	adc								; add offset (Ghost number)
	SETISAR	GHOST_EATABLE_REG
	lr	A, S							; load ghost_eatable_reg
	nm								; and with memory contents
	bz	level.loop.ghosts.checkDirection
	jmp	level.loop.ghosts.draw.quick


level.loop.ghost.flee.checkDirection:

	; ghost eatable, flee!
	jmp	level.loop.ghost.flee






	; ATTACK AI

level.loop.ghosts.checkDirection:

	pi	valid.direction.and.mask
	ci	$ff
	bnz	level.loop.ghosts.checkDirection.distance
	jmp	level.loop.ghosts.checkDirection.end

level.loop.ghosts.checkDirection.distance:
	; find the X distance, dX (r1)
	SETISAR	GHOST_TMP_X_REG
	lr	A, S
	com
	inc
	SETISAR	PACMAN_X_REG
	as	S
	lr	1, A

	; BLINKY aims directly on pac

	; check which one is the current ghost
	SETISAR	GHOST_NUMBER_REG
	lr	A, S

	IF	switch_inky_ai
	ci	2
	bz	level.loop.ghosts.checkDirection.dX.inky
	ENDIF

	IF	switch_pinky_ai
	ci	3
	bz	level.loop.ghosts.checkDirection.dX.pinky
	ENDIF

	ci	1
	bz	level.loop.ghosts.checkDirection.dX.pokey


	br	level.loop.ghosts.checkDirection.dX.continue

	IF	switch_inky_ai	
level.loop.ghosts.checkDirection.dX.inky:
	; INKY (Ghost 2) is active
	; INKY aims to box in PAC between himself and BLINKY, 
	; planning to have PAC 2 spaces forward of current position

	SETISAR	PACMAN_X_REG
	lr	A, S
	lr	1, A
	SETISAR	PACMAN_DIRECTION_REG
	lis	%0011
	ns	S
	; neither right nor left, do nothing
	bz	level.loop.ghosts.checkDirection.dX.inky.continue
	lis	%0001							; test right direction
	ns	S
	; zero here means pac faces left
	bz	level.loop.ghosts.checkDirection.dX.inky.pac.left		
	; pac faces right, add 5 right of pac-man
	lis	5
	as	1
	lr	1, A
	br	level.loop.ghosts.checkDirection.dX.inky.continue
level.loop.ghosts.checkDirection.dX.inky.pac.left:
	li	$fb
	as	1
	lr	1, A

	; subtract BLINKY's x-coordinate from PAC-MAN's (now stored in r1)
level.loop.ghosts.checkDirection.dX.inky.continue:
	SETISAR	GHOST_REG
	lr	A, S
	com
	inc
	as	1
	lr	3, A
	ni	%10000000
	bz	level.loop.ghosts.checkDirection.dX.inky.positive
	; result was negative, BLINKY is right of PAC
	; subtract the distance from PAC's x-coordinate
	lr	A, 3							; already negative, just add
	as	1							; pac-man's offseted x-coordinate
	lr	1, A
	;  check if we're outside maze
	ci	6
	bt	3,level.loop.ghosts.checkDirection.dX.inky.last.step
	lis	6							; set point of aim to 6 (far left maze)
	lr	1, A
	br	level.loop.ghosts.checkDirection.dX.inky.last.step

level.loop.ghosts.checkDirection.dX.inky.positive:
	; result was positive, BLINKY is left of PAC (or same)
	; add the distance to PAC's x-coordinate
	lr	A, 3
	as	1
	lr	1, A
	; check if we're outside maze
	ci	72
	bt	3,level.loop.ghosts.checkDirection.dX.inky.last.step
	li	72
	lr	1, A

level.loop.ghosts.checkDirection.dX.inky.last.step:
	SETISAR	GHOST_TMP_X_REG
	lr	A, S
	com
	inc
	as	1
	lr	1, A
	br	level.loop.ghosts.checkDirection.dX.continue

	ENDIF

	IF	switch_pinky_ai
level.loop.ghosts.checkDirection.dX.pinky:
	; PINKY (Ghost 3) is active
	; PINKY aims four "spaces" in front of PAC, equal to 10 pixels
	SETISAR	PACMAN_DIRECTION_REG
	lis	%0011
	ns	S
	; neither right nor left, do nothing
	bz	level.loop.ghosts.checkDirection.dX.continue
	lis	%0001
	ns	S
	bz	level.loop.ghosts.checkDirection.dX.pinky.pac.left							; zero here means pac faces left
	; pac faces right, add 10 right of pac-man
	lis	10
	as	1
	lr	1, A
	br	level.loop.ghosts.checkDirection.dX.continue
level.loop.ghosts.checkDirection.dX.pinky.pac.left:
	li	$f6
	as	1
	lr	1, A


	ENDIF


level.loop.ghosts.checkDirection.dX.pokey:
	; POKEY behaves like Blinky unless he's closer than 8 spaces
	; which is equal to 20 pixels in this VES version
	; reads from ROM-table to get highest allowed dY, given a dX


level.loop.ghosts.checkDirection.dX.continue:
	; find shortX (r3) & longX (r4)
	lr	A, 1
	ni	%10000000						; check if negative
	bnz	level.loop.ghosts.checkDirection.dX.negative
level.loop.ghosts.checkDirection.dX.positive:
	; shortX = right, longX = left
	lis	%0001
	lr	3, A
	lis	%0010
	lr	4, A
	br	level.loop.ghosts.checkDirection.dX.end
level.loop.ghosts.checkDirection.dX.negative:
	; shortX = left, longX = right
	lis	%0010
	lr	3, A
	lis	%0001
	lr	4, A
	; also get the absolute value of dX
	lr	A, 1
	com
	inc
	lr	1, A
level.loop.ghosts.checkDirection.dX.end:



	; find the Y distance, dY (r2)
	SETISAR	GHOST_TMP_Y_REG
	lr	A, S
	com
	inc
	SETISAR	PACMAN_Y_REG
	as	S
	lr	2, A


	; BLINKY aims directly on pac, nothing needs to be done there

	; check which one is the current ghost
	SETISAR	GHOST_NUMBER_REG
	lr	A, S
	
	IF	switch_pinky_ai
	ci	3
	bz	level.loop.ghosts.checkDirection.dY.pinky
	ENDIF

	IF	switch_inky_ai
	ci	2
	bz	level.loop.ghosts.checkDirection.dY.inky
	ENDIF

	br	level.loop.ghosts.checkDirection.dY.continue

	IF	switch_inky_ai
level.loop.ghosts.checkDirection.dY.inky:
	; INKY (Ghost 2) is active
	; INKY aims to box in PAC between himelf and BLINKY, with
	; imagining PAC 2 spaces forward of current position
	SETISAR	PACMAN_Y_REG
	lr	A, S
	lr	2, A
	SETISAR	PACMAN_DIRECTION_REG
	lis	%1100
	ns	S
	; if pac direction is neither up nor down, do nothing
	bz	level.loop.ghosts.checkDirection.dY.inky.continue
	lis	%0100						; check for direction down?
	ns	S
	; zero here means pac faces up
	bz	level.loop.ghosts.checkDirection.dY.inky.pac.up						
	; pac faces down, add 5 in front of pac-man
	lis	5
	as	2
	ci	53						; check we're inside the maze
	bt	3,level.loop.ghosts.checkDirection.dY.inky.pac.down.done
	li	53
level.loop.ghosts.checkDirection.dY.inky.pac.down.done:
	lr	2, A
	br	level.loop.ghosts.checkDirection.dY.inky.continue
level.loop.ghosts.checkDirection.dY.inky.pac.up:
	li	$fb
	as	2
	inc							; check we're inside maze
	bt	3,level.loop.ghosts.checkDirection.dY.inky.pac.up.done
	clr
level.loop.ghosts.checkDirection.dY.inky.pac.up.done:
	ai	$ff
	lr	2, A

	; subtract BLINKY's y-coordinate from PAC-MAN's (now stored in r2)
level.loop.ghosts.checkDirection.dY.inky.continue:
	SETISAR	[GHOST_REG + 001]
	lr	A, S
	com
	inc
	as	2
	lr	5, A
	ni	%10000000
	bz	level.loop.ghosts.checkDirection.dY.inky.positive
	; result was negative, BLINKY is below PAC
	; subtract the distance from PAC's y-coordinate
	lr	A, 5							; already negative, just add
	as	2							; add pac-man's offseted y-coordinate
	lr	2, A
	;  check if we're outside maze
	ni	%10000000
	bz	level.loop.ghosts.checkDirection.dY.inky.last.step
	clr
	com								; set point of aim to $ff (top of maze)
	lr	2, A
	br	level.loop.ghosts.checkDirection.dY.inky.last.step

level.loop.ghosts.checkDirection.dY.inky.positive:
	; result was positive, BLINKY is above PAC (or same)
	; add the distance to PAC's y-coordinate
	lr	A, 5
	as	2
	lr	2, A
	; check if we're outside maze
	ci	53
	bt	3,level.loop.ghosts.checkDirection.dY.inky.last.step
	li	53
	lr	2, A

level.loop.ghosts.checkDirection.dY.inky.last.step:
	; Goal position - INKY's position
	SETISAR	GHOST_TMP_Y_REG
	lr	A, S
	com
	inc
	as	2
	lr	2, A
	br	level.loop.ghosts.checkDirection.dY.continue

	ENDIF


	IF	switch_pinky_ai

level.loop.ghosts.checkDirection.dY.pinky:	
	; PINKY (Ghost 3) is active
	; PINKY aims four spaces in front of PAC, equal to 10 pixels
	SETISAR	PACMAN_DIRECTION_REG
	lis	%1100
	ns	S
	; if pac direction is neither up nor down, do nothing
	bz	level.loop.ghosts.checkDirection.dY.continue
	lis	%1000
	ns	S
	bz	level.loop.ghosts.checkDirection.dY.pac.up							; zero here means pac faces up
	; pac faces down, add 10 beneath pac-man
	lis	10
	as	2
	lr	2, A
	br	level.loop.ghosts.checkDirection.dY.continue
level.loop.ghosts.checkDirection.dY.pac.up:
	li	$f6
	as	2
	lr	2, A

	ENDIF

level.loop.ghosts.checkDirection.dY.pokey:
	; POKEY behaves like Blinky unless he's closer than 8 spaces
	; which is equal to 20 pixels in this VES version
	; reads from ROM-table to get highest allowed dY, given a dX

level.loop.ghosts.checkDirection.dY.continue:
	; find shortY (r5) & longY (r6)
	lr	A, 2
	ni	%10000000						; check if negative
	bnz	level.loop.ghosts.checkDirection.dY.negative
level.loop.ghosts.checkDirection.dY.positive:
	; shortY = down, longY = up
	lis	%0100
	lr	5, A
	lis	%1000
	lr	6, A
	br	level.loop.ghosts.checkDirection.dY.end
level.loop.ghosts.checkDirection.dY.negative:
	; shortY = up, longY = down
	lis	%1000
	lr	5, A
	lis	%0100
	lr	6, A
	; also get the absolute value of dY
	lr	A, 2
	com
	inc
	lr	2, A
level.loop.ghosts.checkDirection.dY.end:

	SETISAR	GHOST_NUMBER_REG
	lr	A, S
	ci	1
	bnz	level.loop.ghosts.checkDirection.compare.dX.dY

level.loop.ghosts.checkDirection.dY.pokey.tablelookup:
	; read ROM-table, and compare dY with allowed value
	lr	A, 2								; load offset
	ci	20
	; negative value means out of range, behave as Blinky
	bm	level.loop.ghosts.checkDirection.compare.dX.dY
	lr	A, 1
	ci	20
	bm	level.loop.ghosts.checkDirection.compare.dX.dY
	dci	pokey.table							; set read adress
	adc									; add offset (r1)
	lm									; read from ROM
	com									; make negative
	inc
	as	2								; add dY
	; if positive, not close enough
	bp	level.loop.ghosts.checkDirection.compare.dX.dY
	jmp	level.loop.ghosts.quadrant.move.checkDirection

level.loop.ghosts.checkDirection.compare.dX.dY:
	; check if the X distance is greater than the Y distance
	lr	A, 1
	com
	inc
	as	2
	bm	level.loop.ghosts.checkDirection.compare.gt
	; dX <= dY
level.loop.ghosts.checkDirection.compare.lt:
	; shortX
	lr	A, 3
	ns	0
	bnz	level.loop.ghosts.checkDirection.compare.end
	; shortY
	lr	A, 5
	ns	0
	bnz	level.loop.ghosts.checkDirection.compare.end
	; longX
	lr	A, 4
	ns	0
	bnz	level.loop.ghosts.checkDirection.compare.end
	; longY
	lr	A, 6
	ns	0
	bnz	level.loop.ghosts.checkDirection.compare.end
	
	; panic! this point should never be reached
	br	level.loop.ghosts.checkDirection.end
	
	; dX >= dY
level.loop.ghosts.checkDirection.compare.gt:
	; shortY
	lr	A, 5
	ns	0
	bnz	level.loop.ghosts.checkDirection.compare.end
	; shortX
	lr	A, 3
	ns	0
	bnz	level.loop.ghosts.checkDirection.compare.end
	; longY
	lr	A, 6
	ns	0
	bnz	level.loop.ghosts.checkDirection.compare.end
	; longX
	lr	A, 4
	ns	0
	bnz	level.loop.ghosts.checkDirection.compare.end
	
	; panic! this point should never be reached
	br	level.loop.ghosts.checkDirection.end

level.loop.ghosts.checkDirection.compare.end:
	; save the ghost's direction
	SETISAR	GHOST_TMP_DIR_REG
	lr	S, A
	
level.loop.ghosts.checkDirection.end:

level.loop.ghosts.clear:
	; clear the ghost
	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	SETISAR	GHOST_TMP_X_REG
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	; set sprite index
	clr
	lr	4, A
	; set bitmask
;	li	%11111000
	lis	%0111
	com
	lr	5, A
	; draw the sprite
	pi	sprite.draw


	; clear first time blue indicator
	SETISAR	GHOST_TMP_DIR_REG
	lr	A, S
	ni	%11011111
	lr	S, A




level.loop.ghosts.redrawPellets:
	; redraw overlapped pellets
	SETISAR	GHOST_TMP_X_REG
	lr	A, I
	lr	1, A
	lr	A, D
	lr	2, A
	pi	ghost.checkPelletCollision
	; redraw overlapped powerpellets
	SETISAR	GHOST_TMP_X_REG
	lr	A, I
	lr	1, A
	lr	A, D
	lr	2, A
	pi	powerPellets.checkCollision

level.loop.ghosts.move:
	; move the ghosts using the direction set in r40

	SETISAR	GHOST_TMP_DIR_REG
	lr	A, S
	; right (%0001)
	sr	1
	bz	level.loop.ghosts.move.right
	; left (%0010)
	sr	1
	bz	level.loop.ghosts.move.left
	; down (%0100)
	sr	1
	bz	level.loop.ghosts.move.down
	; up (%1000)
	sr	1
	bz	level.loop.ghosts.move.up
	; none (%0000)
	br	level.loop.ghosts.move.end

level.loop.ghosts.move.up:
	; subtract y by 1
	SETISAR	GHOST_TMP_Y_REG
	ds	S						; subtract 1
	br	level.loop.ghosts.move.end
level.loop.ghosts.move.right:
	; add x by 1
	SETISAR	GHOST_TMP_X_REG
	lr	A, S
	inc							; add 1
	lr	S, A
	br	level.loop.ghosts.move.end
level.loop.ghosts.move.down:
	; add y by 1
	SETISAR	GHOST_TMP_Y_REG
	lr	A, S
	inc							; add 1
	lr	S, A
	br	level.loop.ghosts.move.end
level.loop.ghosts.move.left:
	; subtract x by 1
	SETISAR	GHOST_TMP_X_REG
	ds	S						; subtract 1
	br	level.loop.ghosts.move.end
level.loop.ghosts.move.end:

	; check if the ghost has moved through the tunnel
level.loop.ghosts.move.checkTunnel:
	; check Y position
	SETISAR GHOST_TMP_Y_REG
	lr	A, D
	ci	$17
	bnz	level.loop.ghosts.move.checkTunnel.end
	; check X position
	lr	A, S
	ci	$48
	bz	level.loop.ghosts.move.checkTunnel.warpRight	
	; the ghost has passed through left side
	
	ci	$6
	bnz	level.loop.ghosts.move.checkTunnel.end

level.loop.ghosts.move.checkTunnel.warpLeft:
	; set new position on right side
	li	$47
	lr	S, A
	br	level.loop.ghosts.move.checkTunnel.end	
level.loop.ghosts.move.checkTunnel.warpRight:
	; set new position on left side
	lis	$7
	lr	S, A
level.loop.ghosts.move.checkTunnel.end:
	jmp	level.loop.ghosts.draw


level.loop.ghosts.draw.quick:
	; clear the ghost
	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	SETISAR	GHOST_TMP_X_REG
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	; set sprite index
	clr
	lr	4, A
	; set bitmask
;	li	%11111000
	lis	%0111
	com
	lr	5, A
	; draw the sprite
	pi	sprite.draw

level.loop.ghosts.draw.quick.checkPellets:
	; redraw overlapped pellets
	SETISAR	GHOST_TMP_X_REG
	lr	A, I
	lr	1, A
	lr	A, D
	lr	2, A
	pi	ghost.checkPelletCollision
	; redraw overlapped powerpellets
	SETISAR	GHOST_TMP_X_REG
	lr	A, I
	lr	1, A
	lr	A, D
	lr	2, A
	pi	powerPellets.checkCollision
	jmp 	level.loop.ghosts.draw




;**************************************************************************
; eatable ghosts fleeing
;**************************************************************************
level.loop.ghost.flee:


	; check that current direction is valid


	pi	valid.direction.and.mask
	ci	$ff							; we get $ff if not in intersection
	bnz	level.loop.ghost.flee.checkDirection.distance


	; first time in this blue-time?
	; no - branch to not.first

	SETISAR	GHOST_TMP_DIR_REG
	lr	A, S
	ni	%00100000						; mask bit 5 - first time blue indicator
	bz	level.loop.ghost.flee.not.first

	lr	A, S
	ni	%1100							; up/down?
	bnz	level.loop.ghost.flee.first.up.or.down

	; we're moving right or left
	ni	%0010							; moving left?
	bz	level.loop.ghost.flee.first.right
	
	; we're moving left
	lr	A, S
	ni	%11110000						; zero direction
	oi	%00000001						; change to right
	lr	S, A
	br	level.loop.ghost.flee.not.first

level.loop.ghost.flee.first.right:

	; we're moving right
	lr	A, S
	ni	%11110000						; zero direction
	oi	%00000010						; change to left
	lr	S, A
	br	level.loop.ghost.flee.not.first


level.loop.ghost.flee.first.up.or.down:


	ni	%1000							; moving up?
	bz	level.loop.ghost.flee.first.down
	
	; we're moving up
	lr	A, S
	ni	%11110000						; clear direction
	oi	%00000100						; change to moving down
	lr	S, A
	br	level.loop.ghost.flee.not.first
	
level.loop.ghost.flee.first.down:

	; we're moving down
	lr	A, S
	ni	%11110000						; clear direction
	oi	%00001000						; change to moving up
	lr	S, A


	; all done...





level.loop.ghost.flee.not.first:

	jmp	level.loop.ghosts.checkDirection.end		; don't change direction





	; calculate point away from PAC-MAN, set that x and y coordinate



level.loop.ghost.flee.checkDirection.distance:


	; find the X distance to Pac-Man, dX (r1)
	SETISAR	GHOST_TMP_X_REG
	lr	A, S
	com
	inc
	SETISAR	PACMAN_X_REG
	as	S

	SETISAR	GHOST_TMP_X_REG

	com
	inc
	as	S
	lr	1, A
	


	ni	%10000000						; check if negative
	bnz	level.loop.ghost.flee.checkDirection.dX.negative
level.loop.ghost.flee.checkDirection.dX.positive:
	; shortX = right, longX = left
	lis	%0001
	lr	3, A
	lis	%0010
	lr	4, A
	br	level.loop.ghost.flee.checkDirection.dX.end
level.loop.ghost.flee.checkDirection.dX.negative:
	; shortX = left, longX = right
	lis	%0010
	lr	3, A
	lis	%0001
	lr	4, A
	; also get the absolute value of dX
	lr	A, 1
	com
	inc
	lr	1, A
level.loop.ghost.flee.checkDirection.dX.end:




	; find the Y distance to Pac-Man, dY (r2)

	SETISAR	GHOST_TMP_Y_REG
	lr	A, S
	com
	inc
	SETISAR	PACMAN_Y_REG
	as	S

	SETISAR	GHOST_TMP_Y_REG

	com
	inc
	as	S
	lr	2, A



	ni	%10000000						; check if negative
	bnz	level.loop.ghost.flee.checkDirection.dY.negative
level.loop.ghost.flee.checkDirection.dY.positive:
	; shortY = down, longY = up
	lis	%0100
	lr	5, A
	lis	%1000
	lr	6, A
	br	level.loop.ghost.flee.checkDirection.dY.end
level.loop.ghost.flee.checkDirection.dY.negative:
	; shortY = up, longY = down
	lis	%1000
	lr	5, A
	lis	%0100
	lr	6, A
	; also get the absolute value of dY
	lr	A, 2
	com
	inc
	lr	2, A
level.loop.ghost.flee.checkDirection.dY.end:





	jmp	level.loop.ghosts.checkDirection.compare.dX.dY

;**************************************************************************
; dead ghost returning to nest
;**************************************************************************

level.loop.ghosts.return.nest:

	; check if ghost is "nested"
	SETISAR	GHOST_TMP_DIR_REG
	lr	A, S
	ni	%10000000						; Check if ghost is nested
	bnz	level.loop.ghosts.return.nest.unnest

    
	; check if ghost has returned to start position
	SETISAR	GHOST_TMP_X_REG
	lr	A, I
	ci	39
	bnz	level.loop.ghosts.return.nest.checkDirection
	lr	A, S
	ci	17
	bnz	level.loop.ghosts.return.nest.checkDirection

level.loop.ghosts.return.nest.unnest:
	pi	ghost.nest
	jmp	level.loop.ghosts.draw

level.loop.ghosts.return.nest.checkDirection:

	pi	valid.direction.and.mask
	ci	$ff
	bnz	level.loop.ghosts.return.nest.checkDirection.distance
	jmp	level.loop.ghosts.checkDirection.end

level.loop.ghosts.return.nest.checkDirection.distance:
	; find the X distance, dX (r1)
	SETISAR	GHOST_TMP_X_REG
	lr	A, S
	com
	inc
	ai	39							; x coordinate for startposition
	lr	1, A
	; find shortX (r3) & longX (r4)
	ni	%10000000						; check if negative
	bnz	level.loop.ghosts.return.nest.checkDirection.dX.negative
level.loop.ghosts.return.nest.checkDirection.dX.positive:
	; shortX = right, longX = left
	lis	%0001
	lr	3, A
	lis	%0010
	lr	4, A
	br	level.loop.ghosts.return.nest.checkDirection.dX.end
level.loop.ghosts.return.nest.checkDirection.dX.negative:
	; shortX = left, longX = right
	lis	%0010
	lr	3, A
	lis	%0001
	lr	4, A
	; also get the absolute value of dX
	lr	A, 1
	com
	inc
	lr	1, A
level.loop.ghosts.return.nest.checkDirection.dX.end:

	; find the Y distance, dY (r2)
	SETISAR	GHOST_TMP_Y_REG
	lr	A, S
	com
	inc
	ai	17							; y coordinate for startposition
	lr	2, A
	; find shortY (r5) & longY (r6)
	ni	%10000000						; check if negative
	bnz	level.loop.ghosts.return.nest.checkDirection.dY.negative
level.loop.ghosts.return.nest.checkDirection.dY.positive:
	; shortY = down, longY = up
	lis	%0100
	lr	5, A
	lis	%1000
	lr	6, A
	br	level.loop.ghosts.return.nest.checkDirection.dY.end
level.loop.ghosts.return.nest.checkDirection.dY.negative:
	; shortY = up, longY = down
	lis	%1000
	lr	5, A
	lis	%0100
	lr	6, A
	; also get the absolute value of dY
	lr	A, 2
	com
	inc
	lr	2, A
level.loop.ghosts.return.nest.checkDirection.dY.end:
	
	; jump to original code
	jmp	level.loop.ghosts.checkDirection.compare.dX.dY

 

;**************************************************************************
; Time in seconds for waves of retreat or attack mode, starting with retreat
; 7, 20, 7, 20, 5, 20, 5, infinite
; 
; ghosts returning to their quadrants 
;**************************************************************************

level.loop.ghosts.quadrant.move:
	SETISAR	ATTACK_COUNTER_REG    
	clr
	as	S
	bz	level.loop.ghosts.quadrant.move.long	
	ci	2
	bz	level.loop.ghosts.quadrant.move.long	
	SETISAR	GLOBAL_TIMER_REG
	lr	A, S
	ci	$BD
	bm	level.loop.ghosts.quadrant.move.checkDirection
	br	level.loop.ghosts.quadrant.move.timers
level.loop.ghosts.quadrant.move.long:	
	SETISAR	GLOBAL_TIMER_REG
	lr	A, S
	ci	$AA
	bm	level.loop.ghosts.quadrant.move.checkDirection
level.loop.ghosts.quadrant.move.timers:
	SETISAR	ATTACK_COUNTER_REG			; increase attack counter
	lis	1
	as	S
	lr	S, A
	; clear global_timer
	SETISAR	GLOBAL_TIMER_REG
	clr
	lr	S, A							; clear global timer and update
	ds	S

level.loop.ghosts.quadrant.move.checkDirection:
	pi	valid.direction.and.mask
	ci	$ff
	bnz	level.loop.ghosts.quadrant.move.checkDirection.x.ghost
	jmp	level.loop.ghosts.checkDirection.end

level.loop.ghosts.quadrant.move.checkDirection.x.ghost:
	; find the X distance, dX (r1)
	SETISAR	GHOST_NUMBER_REG
	lr	A, S
	dci	ghost.retreat.x.data
	adc
	lm	
level.loop.ghosts.quadrant.move.checkDirection.x.ghost.set:
	lr	3, A								; save temp. in r3
	SETISAR	GHOST_TMP_X_REG
	lr	A, S
	com
	inc
	as	3
	lr	1, A
	; find shortX (r3) & longX (r4)
	ni	%10000000						; check if negative
	bnz	level.loop.ghosts.quadrant.move.checkDirection.dX.negative
level.loop.ghosts.quadrant.move.checkDirection.dX.positive:
	; shortX = right, longX = left
	lis	%0001
	lr	3, A
	lis	%0010
	lr	4, A
	br	level.loop.ghosts.quadrant.move.checkDirection.dX.end
level.loop.ghosts.quadrant.move.checkDirection.dX.negative:
	; shortX = left, longX = right
	lis	%0010
	lr	3, A
	lis	%0001
	lr	4, A
	; also get the absolute value of dX
	lr	A, 1
	com
	inc
	lr	1, A
level.loop.ghosts.quadrant.move.checkDirection.dX.end:

	; find the Y distance, dY (r2)
	SETISAR	GHOST_NUMBER_REG
	lr	A, S
	dci	ghost.retreat.y.data
	adc
	lm
level.loop.ghosts.quadrant.move.checkDirection.y.ghost.set:
	lr	5, A								; save tmp in r5

	SETISAR	GHOST_TMP_Y_REG
	lr	A, S
	com
	inc
	as	5
	lr	2, A
	; find shortY (r5) & longY (r6)
	ni	%10000000						; check if negative
	bnz	level.loop.ghosts.quadrant.move.checkDirection.dY.negative
level.loop.ghosts.quadrant.move.checkDirection.dY.positive:
	; shortY = down, longY = up
	lis	%0100
	lr	5, A
	lis	%1000
	lr	6, A
	br	level.loop.ghosts.quadrant.move.checkDirection.dY.end
level.loop.ghosts.quadrant.move.checkDirection.dY.negative:
	; shortY = up, longY = down
	lis	%1000
	lr	5, A
	lis	%0100
	lr	6, A
	; also get the absolute value of dY
	lr	A, 2
	com
	inc
	lr	2, A
level.loop.ghosts.quadrant.move.checkDirection.dY.end:

     	; jump back to original code
	jmp	level.loop.ghosts.checkDirection.compare.dX.dY


;**************************************************************************
	; draw a ghost
level.loop.ghosts.draw:

;**************************************************************************


	; check if any ghost is eaten
	SETISAR	GHOST_EATABLE_REG
	lr	A, S
	sr	4
	bz	level.loop.ghosts.draw.ppcheck			; none eaten, next check

	; current ghost already eaten?

	SETISAR	GHOST_NUMBER_REG
	lr	A, S
	lisl	4
	ci	3
	bz	level.loop.ghosts.draw.return.nest.ghost3
	ci	2
	bz	level.loop.ghosts.draw.return.nest.ghost2
	ci	1
	bz	level.loop.ghosts.draw.return.nest.ghost1
 	; ghost 0 returned
 	lr	A, S
 	ni	%00010000
	bnz	level.loop.ghosts.draw.eaten
   	br	level.loop.ghosts.draw.ppcheck
level.loop.ghosts.draw.return.nest.ghost1:
 	lr	A, S
 	ni	%00100000
	bnz	level.loop.ghosts.draw.eaten
  	br	level.loop.ghosts.draw.ppcheck
level.loop.ghosts.draw.return.nest.ghost2:
 	lr	A, S
 	ni	%01000000
	bnz	level.loop.ghosts.draw.eaten
   	br	level.loop.ghosts.draw.ppcheck	
level.loop.ghosts.draw.return.nest.ghost3:
 	lr	A, S
 	ni	%10000000
	bnz	level.loop.ghosts.draw.eaten

level.loop.ghosts.draw.ppcheck:

	SETISAR	POWERPELLETS_REG				; quick check to see if p.pellet active
	lis	%0010							; check if slow ghost bit (5) is 1
	sl	4
	ns	S
	bz	level.loop.ghosts.draw.normal			; if not, draw as normal

	; check if current ghost is eatable
	SETISAR	GHOST_NUMBER_REG
	lr	A, S							; load ghost number into A
	dci	eatable.mask.data					; set DC
	adc								; add offset
	lm								; load data from ROM
	lisl	4							; set eatable register
	ns	S							; AND register
	bz	level.loop.ghosts.draw.normal			; if current ghost is 0 - not eatable
	lr	A, I							; increase ISAR to slow_timer_reg
	lr	A, S							; flash near the end
	ni	%10001100	
	bnz	level.loop.ghosts.draw.ghost.eatable
	br	level.loop.ghosts.draw.normal			; flash normal colors

level.loop.ghosts.draw.ghost.eatable:

	; draw ghosts with other sprite and different color
	li	blue
	lr	1, A
	; load x coordinate
	SETISAR	GHOST_TMP_X_REG
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, I
	lr	3, A
	; set sprite index
	lr	A, I						; increase ISAR to skip direction reg
	lr	A, S
	ni	%1
	ai	24
	lr	4, A

	br	level.loop.ghosts.draw.eatable

level.loop.ghosts.draw.eaten:

	; draw ghosts with other sprite and different color
	li	blue
	lr	1, A
	; load x coordinate
	SETISAR	GHOST_TMP_X_REG
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, I
	lr	3, A
	; set sprite index
	li	26
	lr	4, A

	br	level.loop.ghosts.draw.eatable

level.loop.ghosts.draw.normal:

	dci	gfx.ghost.colors
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	adc
	lm
	lr	1, A
	; load x coordinate
	SETISAR	GHOST_TMP_X_REG
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, I
	lr	3, A
	; set sprite index
	lr	A, I						; increase ISAR to skip direction reg
	lr	A, S
	ni	%1
	ai	17
	lr	4, A

level.loop.ghosts.draw.eatable:
	; set bitmask
;	li	%11111000
	lis	%0111
	com
	lr	5, A
	; draw the sprite
	pi	sprite.draw

level.loop.ghosts.draw.end:

	; decrease the animation index
	SETISAR	GHOST_TMP_ANIM_REG
	ds	S

	; restore modified registers
	SETISAR	GHOST_TMP_REG
	; load registers
	lr	A, I
	lr	0, A
	lr	A, I
	lr	1, A
	lr	A, I
	lr	2, A
	lr	A, S
	lr	3, A
	; get ISAR location for ghost
	SETISAR GHOST_NUMBER_REG
	; multiply by 4 bytes per ghost
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	IS, A
	; store at temp ghost position
	lr	A, 0
	lr	I, A
	lr	A, 1
	lr	I, A
	lr	A, 2
	lr	I, A
	lr	A, 3
	lr	S, A

level.loop.ghosts.checkLoop:
	; check if we've reached the last ghost
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	inc
	ci	GHOST_NUMBER
	bz	level.loop.ghosts.end
	; update next ghost
	lr	S, A
	jmp	level.loop.ghosts

level.loop.ghosts.end:

;---------------------------------------------------------------------------
; Pac-man ghost collision
;---------------------------------------------------------------------------
; check if pacman has collided with a ghost

	; only check for collisions every second loop

	SETISAR	GLOBAL_TIMER_REG
	lr	A, S
	ni	%00000001
	bz	level.loop.checkPacmanCollision.start
	jmp	level.loop.checkPacmanCollision.end

level.loop.checkPacmanCollision.start:
	; load first ghost
	SETISAR GHOST_NUMBER_REG
	clr
	lr	S, A
level.loop.checkPacmanCollision:
	; load this ghost's coordinates
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	IS, A
	; x position
	lr	A, I
	lr	0, A
	; y position
	lr	A, S
	lr	1, A

level.loop.checkPacmanCollision.x:
	; check the X axis
	lr	A, 0
	com
	inc
	SETISAR	PACMAN_X_REG
	as	S
	bm	level.loop.checkPacmanCollision.x.mi
	; positive
	com
	inc
	ai	DEATH_DISTANCE
	bm	level.loop.checkPacmanCollision.checkLoop.jump
	br	level.loop.checkPacmanCollision.y
	; negative
level.loop.checkPacmanCollision.x.mi
	ai	DEATH_DISTANCE
	bm	level.loop.checkPacmanCollision.checkLoop.jump

level.loop.checkPacmanCollision.y:
	; check the Y axis
	lr	A, 1
	com
	inc
	SETISAR	PACMAN_Y_REG
	as	S
	bm	level.loop.checkPacmanCollision.y.mi
	; positive
	com
	inc
	ai	DEATH_DISTANCE
	bm	level.loop.checkPacmanCollision.checkLoop.jump
	br	level.loop.checkPacmanCollision.collided
	; negative
level.loop.checkPacmanCollision.y.mi
	ai	DEATH_DISTANCE
	bm	level.loop.checkPacmanCollision.checkLoop.jump

	; pacman has collided with a ghost
level.loop.checkPacmanCollision.collided:

; *****************************************************************************
; branch out of range

	br	level.loop.checkPacmanCollision.skipjump.check.eaten
level.loop.checkPacmanCollision.checkLoop.jump:
	jmp	level.loop.checkPacmanCollision.checkLoop


level.loop.checkPacmanCollision.skipjump.check.eaten:

	; first check if any ghost is returning to nest
	SETISAR	GHOST_EATABLE_REG
	li	%11110000
	ns	S
	bz	level.loop.checkPacmanCollision.skipjump		; no ghost currently eaten

	SETISAR	GHOST_NUMBER_REG
	lr	A, S							; load ghost number in A [0,3]
	dci	eatable.mask.data					; set table adress
	adc								; add ghostnumber to DC
	lm								; read data to A
	sl	4							; shift left  <= LLLL0000
	lisl	4							; check "is eaten"
	ns	S							; if not zero, ghost already eaten
	bnz	level.loop.checkPacmanCollision.checkLoop.jump

	; pacman dies if ghost aren't eatable
level.loop.checkPacmanCollision.skipjump:

	; check if current ghost is eatable
	SETISAR	GHOST_NUMBER_REG
	lr	A, S							; load Ghost number in A
	dci	eatable.mask.data					; set DC
	adc								; add offset
	lisl	4
	lis	15							; mask upper nibble
	ns	S							; AND register
	nm								; AND data from RAM
	bnz	skip.to.here					; ghost was eatable
	jmp	level.loop.checkPacmanCollision.collided.endLevel

skip.to.here:

	; ghost is eatable...
	com								; invert bitpattern
	ns	S							; AND to S to reset "ghost_eatable"
	lr	S, A


	; clear the ghost
	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	IS, A						; set ISAR for correct ghost
	; load x coordinate
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	; set sprite index
	clr
	lr	4, A
	; set bitmask
	lis	%0111
	com
	lr	5, A
	; draw the sprite
	pi	sprite.draw

	IF	switch_music
	;play sound
	dci	sfx.ghostEat
	pi	playSong
	ENDIF

	; clear space to plot the ghost eat score

	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	IS, A						; set ISAR for correct ghost
	; load x coordinate
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	SETISAR PACMAN_X_REG
	lr	A, S
	ci	43						; don't plot score furthest to the right
	bp	level.loop.checkPacmanCollision.collided.plot.score0
	lr	A, 2
	ai	$fa						; change X-position
	lr	2, A
level.loop.checkPacmanCollision.collided.plot.score0:
	clr
	lr	4, A						; erase-sprite
	; set bitmask
	lis	%0111
	com
	lr	5, A
	; draw the sprite
	pi	sprite.draw


level.loop.checkPacmanCollision.collided.add.score:
	; add to the score
	li	%01100110
	lr	Ku, A
	lr	Kl, A
	clr
	lr	6, A
	SETISAR	GHOST_EATABLE_REG
	lr	A, S
	ni	%00001111						; reset upper nibble
	bz	level.loop.checkPacmanCollision.collided.add.score.1600	; last ghost if 0
	ni	%00000001						; mask off last bit
	as	6							; add result to r6
	lr	6, A
	lr	A, S
	ni	%00001111						; reset upper nibble
	sr	1
	ni	%00000001
	as	6
	lr	6, A
	lr	A, S
	ni	%00001111						; reset upper nibble
	sr	1
	sr	1
	ni	%00000001
	as	6
	lr	6, A
	lr	A, S
	ni	%00001111						; reset upper nibble
	sr	1
	sr	1
	sr	1
	ni	%00000001
	as	6
	lr	6, A

	; there's now 1, 2 or 3 in r6 
	ci	1
	bz	level.loop.checkPacmanCollision.collided.add.score.800
	ci	2
	bz	level.loop.checkPacmanCollision.collided.add.score.400

	; if it's not 1 or 2 it has to be 3, score is 200
	li	30
	lr	4, A
	li	%01101000					; 200
	br	level.loop.checkPacmanCollision.collided.add.score.done
level.loop.checkPacmanCollision.collided.add.score.800:
	li	28
	lr	4, A
	li	%01101110					; 800
	br	level.loop.checkPacmanCollision.collided.add.score.done
level.loop.checkPacmanCollision.collided.add.score.400:
	li	29
	lr	4, A
	li	%01101010					; 400
	br	level.loop.checkPacmanCollision.collided.add.score.done	
level.loop.checkPacmanCollision.collided.add.score.1600:
	li	27
	lr	4, A
	li	%01111100					; 1600
level.loop.checkPacmanCollision.collided.add.score.done:
	lr	Qu, A
	li	%01100110
	lr	Ql, A
	pi	score.add

	; plot eat ghost score


	li	EATSCORE_COLOR
	lr	1, A
	; load x coordinate
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	IS, A						; set ISAR for correct ghost
	; load x coordinate
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	SETISAR PACMAN_X_REG
	lr	A, S
	ci	43						; don't plot score furthest to the right
	bp	level.loop.checkPacmanCollision.collided.plot.score1
	lr	A, 2
	ai	$fa						; change X-position
	lr	2, A
level.loop.checkPacmanCollision.collided.plot.score1:
	; sprite index already set
	; set bitmask
	lis	%0111
	com
	lr	5, A
	; draw the sprite
	pi	sprite.draw



	; clear background for score plot

	; clear the ghost
	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	IS, A						; set ISAR for correct ghost
	; load x coordinate
	lr	A, I
	ai	5
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	; set sprite index
	clr
	lr	4, A
	; set bitmask
;	li	%11111000
	lis	%0111
	com
	lr	5, A
	SETISAR PACMAN_X_REG
	lr	A, S
	ci	43						; don't plot score furthest to the right
	bp	level.loop.checkPacmanCollision.collided.plot.score2
	lr	A, 2
	ai	$fa						; change X-position
	lr	2, A
level.loop.checkPacmanCollision.collided.plot.score2:
	; draw the sprite
	pi	sprite.draw

	li	EATSCORE_COLOR
	lr	1, A
	; load x coordinate
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	IS, A						; set ISAR for correct ghost
	; load x coordinate
	lr	A, I
	ai	6
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	; sprite 00
	li	31
	lr	4, A
	lis	%0111
	com
	lr	5, A
	SETISAR PACMAN_X_REG
	lr	A, S
	ci	43						; don't plot score furthest to the right
	bp	level.loop.checkPacmanCollision.collided.plot.score3
	lr	A, 2
	ai	$fa						; change X-position
	lr	2, A
level.loop.checkPacmanCollision.collided.plot.score3:
	; draw the sprite
	pi	sprite.draw



	; check for extra life


	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%10000000
	bz	level.loop.checkPacmanCollision.collided.noextralife

	SETISAR	PACMAN_LIVES_REG
	lis	1
	as	S
	sl	1							; remove extra life marker
	sr	1
	lr	S, A							; store pacman_lives_reg increased	
	IF switch_music
	; play the extra life sound
	dci	sfx.extraPacman
	pi	playSong
	ENDIF


	; draw pacman lives

	pi	pacman.drawLives


level.loop.checkPacmanCollision.collided.noextralife:

	; pause!!

	li	$a0
	lr	5, A
	pi	delay

	; repair score-plot damage

	; erase score plots

	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	IS, A						; set ISAR for correct ghost
	; load x coordinate
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	; set sprite index
	clr
	lr	4, A
	; set bitmask
	lis	%0111
	com
	lr	5, A
	SETISAR PACMAN_X_REG
	lr	A, S
	ci	43						; don't plot score furthest to the right
	bp	level.loop.checkPacmanCollision.collided.plot.score4
	lr	A, 2
	ai	$fa						; change X-position
	lr	2, A
level.loop.checkPacmanCollision.collided.plot.score4:
	; draw the sprite
	pi	sprite.draw

	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	IS, A						; set ISAR for correct ghost
	; load x coordinate
	lr	A, I
	ai	6
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	; set sprite index
	clr
	lr	4, A
	; set bitmask
	lis	%0111
	com
	lr	5, A
	SETISAR PACMAN_X_REG
	lr	A, S
	ci	43						; don't plot score furthest to the right
	bp	level.loop.checkPacmanCollision.collided.plot.score5
	lr	A, 2
	ai	$fa						; change X-position
	lr	2, A
level.loop.checkPacmanCollision.collided.plot.score5:
	; draw the sprite
	pi	sprite.draw

	; draw the map
	; set map bg color
	; color 1
	li	clear
	lr	1, A						; r1
	pi	map.draw
	pi	pacman.draw

	; set ghost to eaten until it has returned to nest
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	dci	eaten.nest.data
	adc
	lisl	4						; ghost_eatable_reg
	lr	A, S
	om
	lr	S, A


	; move ghost

level.loop.checkPacmanCollision.collided.moveGhost:
level.loop.checkPacmanCollision.collided.change.ghost.position:

	; redraw all pellets in case of partial collision
	pi	powerPellets.draw
	pi	pellets.draw

	br	level.loop.checkPacmanCollision.end

level.loop.checkPacmanCollision.collided.endLevel:
	jmp	level.ended

level.loop.checkPacmanCollision.checkLoop:
	; check if we've reached the last ghost
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	inc
	ci	GHOST_NUMBER
	bz	level.loop.checkPacmanCollision.end
	; update next ghost
	lr	S, A
	jmp	level.loop.checkPacmanCollision

level.loop.checkPacmanCollision.end:

;---------------------------------------------------------------------------
; pellets
;---------------------------------------------------------------------------

level.loop.checkPellets:
	; check if pacman has eaten any pellets
	pi	pacman.checkPelletCollision

	; check if we collided with a powerpellet
	SETISAR	POWERPELLETS_REG
	lr	A, S
	oi	%00010000						; set bit 4 to indicate Pacman collision
	lr	S, A
	SETISAR	PACMAN_X_REG
	lr	A, I
	lr	1, A
	lr	A, D
	lr	2, A
	pi	powerPellets.checkCollision
	SETISAR	POWERPELLETS_REG
	lr	A, S
	ni	%11101111						; reset bit 4
	lr	S, A


	; check if board is clear 

level.loop.checkBoard.quick:
	SETISAR	BONUS_PRIZE_REG
	lr	A, S
	ci	185
	bm	level.loop.checkBoard
	br	level.loop.checkBoard.end
	
level.loop.checkBoard:
	; check to see if the board is cleared
	dci	ram + 3						; change from v b29b

	; load our loop counter
	li	57							; change from v b29b
	lr	0, A
level.loop.checkBoard.loop:
	; decrease counter
	ds	0
	bm	level.loop.checkBoard.empty		; the board has been cleared
	; check the next byte
	lm
	ci	0
	bnz	level.loop.checkBoard.end		; there's still some pellets left, end the loop
	br	level.loop.checkBoard.loop		; continue the loop
	
level.loop.checkBoard.empty:
	; the board has been cleared


	SETISAR	POWERPELLETS_REG
	lr	A, I
	ni	%00001111
	bnz	level.loop.checkBoard.end
	
	jmp	level.cleared

level.loop.checkBoard.end:

;---------------------------------------------------------------------------
; loop end
;---------------------------------------------------------------------------

level.loop.end:
	jmp	level.loop

;===========================================================================
; Level ending functions
;===========================================================================

;---------------------------------------------------------------------------
; level cleared
;---------------------------------------------------------------------------

level.cleared:

	pi	score.draw

	;clear eatable ghosts and set to not eaten
	SETISAR	GHOST_EATABLE_REG
	clr
	lr	S, A

	; erase ghosts

	pi	ghosts.clear

	; draw Pac-Man's final frame
	SETISAR PACMAN_ANIMINDEX_REG
	lis	2						; load the closed-sprite index
	lr	S, A						; save the new index
	pi	pacman.draw					; redraw Pac-Man

	; load loop counter
	lis	4						; how many times to flash the screen
	lr	0, A						; stored in r0

level.cleared.loop:

	; black & white
	li	$ff
	lr	1, A
	; horizontal position (250 = color row)
	li	250
	lr	2, A
	; vertical position
	li	58
	lr	3, A	
level.cleared.loop.on:
	ds	3
	bm	level.cleared.loop.on.end
	pi	plot
	br	level.cleared.loop.on
level.cleared.loop.on.end:

	; delay for a little bit
	li	$5f
	lr	5, A
	pi	delay

	; color, grey background
	lis	$0
	lr	1, A
	; horizontal position (250 = color row)
	li	250
	lr	2, A
	; vertical position
	li	58
	lr	3, A
level.cleared.loop.off:
	ds	3
	bm	level.cleared.loop.off.end
	pi	plot
	br	level.cleared.loop.off
level.cleared.loop.off.end:

	; delay for a little bit
	li	$5f
	lr	5, A
	pi	delay

level.cleared.check:
	ds	0
	bnz	level.cleared.loop


	; removed code since v b29b that increases level counter reg 'O'41
	; which is wrong, the level register is 'O'60
	; this increases ghost 0 Y-register instead


	; delay for a little bit
	li	$8f
	lr	5, A
	pi	delay

level.cleared.end:
	; increase the level number
	SETISAR	LEVEL_NUMBER_REG
	lr	A, S
	inc


	lr	S, A


	; check if there's an intermission
;##############################################################################
	; check for carry -> 256th level
;##############################################################################
;	bc	jump.intermission_256

	ci	3
	bz	jump.intermission
	ci	6
	bz	jump.intermission
	ci	10
	bz	jump.intermission
	ci	14
	bz	jump.intermission
	ci	18	
	bz	jump.intermission
	

	; start the next level
	jmp	level.init


jump.intermission:
	jmp	intermission
;jump.intermission_256:
;	jmp	intermission_256


;---------------------------------------------------------------------------
; level ended
;---------------------------------------------------------------------------

level.ended:

	pi	score.draw

	; short delay
	li	$ff
	lr	5, A
	pi	delay

	pi	ghosts.clear

	; store global timer
	SETISAR	GLOBAL_TIMER_REG
	lr	A, S
	SETISAR	ATTACK_TIMER_REG
	lr	S, A


	; start the death sequence
level.ended.deathSequence:

	; part 1
level.ended.deathSequence.1:
	; clear Pac-Man
	pi	pacman.clear

	; sprite index
	lis	1
	lr	4, A
	li	%10101010
	lr	7, A
	pi	pacman.draw

    IF switch_music
	; play death tune
	dci	sfx.dying.0
	pi	playSong
    ENDIF
   
	; part 2
level.ended.deathSequence.2:
	; clear Pac-Man
	pi	pacman.clear

	; load index
	li	20
	lr	4, A
	; draw the sprite
	li	%10101010
	lr	7, A
	pi	pacman.draw				; redraw Pac-Man

    IF switch_music
	; play death tune
	dci	sfx.dying.1
	pi	playSong
    ENDIF

	; part 3
level.ended.deathSequence.3:
	; clear Pac-Man
	pi	pacman.clear

	; load the sprite index
	li	21
	lr	4, A
	; draw the sprite
	li	%10101010
	lr	7, A
	pi	pacman.draw				; redraw Pac-Man

    IF switch_music
	; play death tune
	dci	sfx.dying.2
	pi	playSong
    ENDIF
   
   	; part 4
level.ended.deathSequence.4:
	; clear Pac-Man
	pi	pacman.clear

	; load the sprite index
	li	22
	lr	4, A
	; draw the sprite
	li	%10101010
	lr	7, A
	pi	pacman.draw				; redraw Pac-Man

    IF switch_music
	; play death tune
	dci	sfx.dying.3
	pi	playSong
    ENDIF

   	; part 5
level.ended.deathSequence.5:
	; clear Pac-Man
	pi	pacman.clear

	; shut off powerpellet
	SETISAR	POWERPELLETS_REG
	lr	A, S
	ni	%11011111
	lr	I, A
	; set all ghosts to not eatable and non-eaten
	clr
	lr	S, A
	; clear prize
	SETISAR	BONUS_PRIZE_ON
	lr	S, A
	pi	prize.draw

level.ended.restart:
	; delay for a little bit
	li	$ff
	lr	5, A
	pi	delay

	; reset the level if he has lives remaining
	SETISAR	PACMAN_LIVES_REG
	ds	S
	lr	A, S
	ni	%00000111
	bz	level.ended.reset

	; swap players if two player mode is on
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%00100000
	bz	level.ended.continue.level		; not two players, continue as usual
	pi	swap.player
	; check that this player has lives left, otherwise swap back
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%00000111
	bnz	level.ended.restart.ok
	pi	swap.player					; change back to player with lives
level.ended.restart.ok:

	; restore global timer after player swap
	SETISAR	ATTACK_TIMER_REG
	clr
	as	S
	bz	level.ended.restart.ok.skip.timer
	SETISAR	GLOBAL_TIMER_REG
	lr	S, A
	SETISAR	ATTACK_TIMER_REG
	clr
	lr	S, A
level.ended.restart.ok.skip.timer:
	jmp	level.init.drawScreen
	; continue the level

level.ended.continue.level:

	; redraw all pellets in case of partial collision
	pi	powerPellets.draw
	pi	pellets.draw

	jmp	level.init.intro

level.ended.reset:

	; draw "PLAYER 1"
	dci	gfx.player1.parameters
	pi	blitGraphic
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%01000000
	bnz	level.ended.reset.player2
	br	level.ended.reset.gameover
level.ended.reset.player2:
	; draw "PLAYER 2"
	dci	gfx.player2.parameters
	pi	blitGraphic

level.ended.reset.gameover:
	; draw "GAME OVER"
	dci	gfx.gameover.parameters
	pi	blitGraphic

	pi	hiscore.draw

	li	$ff
	lr	5, A
	pi	delay
	li	$ff
	lr	5, A
	pi	delay



	; changed the score adding since v b29b




	SETISAR	SCORE_REG
	lr	A, S
	com
	inc								; highest number of score is now negative
	lisl	0
	as	S
	bm	level.ended.reset.updateHS			; if negative new high score
	bnz	level.ended.reset.updateHS.end			; if positive, we have no hiscore
	
	lisl	5
	lr	A, S
	com
	inc
	lisl	1
	as	S
	bm	level.ended.reset.updateHS			; if negative new high score
	bnz	level.ended.reset.updateHS.end			; if positive, we have no hiscore

	
	lisl	6
	lr	A, S
	com
	inc
	lisl	2
	as	S
	bm	level.ended.reset.updateHS			; if negative new high score
	bnz	level.ended.reset.updateHS.end			; if positive, we have no hiscore


	lisl	7
	lr	A, S
	com
	inc
	lisl	3
	as	S
	bm	level.ended.reset.updateHS			; if negative new high score
	bnz	level.ended.reset.updateHS.end			; if positive, we have no hiscore

	
level.ended.reset.updateHS:
	; update high score in RAM
	SETISAR	SCORE_REG
	lr	A, I
	dci	ram + $602
	st
	lr	A, I
	st
	lr	A, I
	st
	lr	A, S
	st


level.ended.reset.updateHS.end:

	; check if other player has any lives left, if so restart level
	SETISAR	PACMAN_LIVES_REG
	lis	%0010
	sl	4
	ns	S						; check if we have two player mode
	bz	level.ended.reset.start.new.level
	pi	swap.player
	; draw the new hiscore
	pi	hiscore.draw

	; restore "hi-score" register
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%01000000
	bz	level.ended.reset.restore.first.player
	dci	ram + $ec
	br	level.ended.reset.restore.other.player
level.ended.reset.restore.first.player:
	dci	ram + $8c
level.ended.reset.restore.other.player:
	SETISAR	HISCORE_REG
	lm
	lr	I, A
	lm
	lr	I, A
	lm
	lr	I, A
	lm
	lr	S, A

	SETISAR	PACMAN_LIVES_REG
	lis	%0111
	ns	S
	bz	level.ended.reset.start.new.level
	jmp	level.init.drawScreen			; restart with other player

	; start a new level
level.ended.reset.start.new.level:			; no player has lives left

	; draw the new hiscore
	pi	hiscore.draw

	; pause
	li	$ff
	lr	5, A
	pi	delay
	li	$ff
	lr	5, A
	pi	delay


	jmp	0						; re-start on intro



;***************************************************************************


;===========================================================================
; Pellet Functions 
;===========================================================================

;---------------------------------------------------------------------------
; Pellet Initalization
;---------------------------------------------------------------------------
; copy the pellet map into memory

pellets.init:
	; load DC1 with the RAM address
	dci	ram
	xdc
	; load DC0 with the map address
	dci	maze.pellets

	; start the copy cycle
	li	60
	lr	0, A
pellets.init.loop:
	; get and save a byte
	lm
	xdc
	st
	xdc

	; check if the loop has ended
	ds	0
	bnz	pellets.init.loop

pellets.init.end:
	pop

;---------------------------------------------------------------------------
; Check Pacman's Intersection 
;---------------------------------------------------------------------------
; checks if pac-man is in an intersection

pacman.checkIntersection:
	SETISAR	PACMAN_X_REG
	; X position
	lr	A, I
	lr	4, A
	; Y position
	lr	A, S
	lr	5, A
	; jump to checkIntersection
	br	checkIntersection

;---------------------------------------------------------------------------
; Check Intersection 
;---------------------------------------------------------------------------
; checks to see if a location is in an intersection (r4 = x, r5 = y)
; if so, it returns with A set to 1 and the intersection boundaries in r0
; else, returns with A set to 0
;
; modifies: ISAR, r0, r1, r2, r3, Q, K
;


checkIntersection:
	; save the return address
	lr	K, P
	pi	kstack.push

	; check to see if we're on an intersection vertically
	dci	ram + $200					; set adress for lookuptable
	lr	A, 5						; load y position
	ai	1						; fix -1 offset of intersections
	adc							; add offset
	clr							; clear A
	am							; add memory, status bits are set
	bz	.checkIntersectionFalse

	; check to see if we're on an intersection horizontally
	dci	ram + $200					; set adress for lookuptable
	lr	A, 4						; load x position
	ci	30						; skip positions not used
	bz	.checkIntersectionFalse
	ci	48
	bz	.checkIntersectionFalse
	ai	%11111010					; subtract 6 to start reading on 0
	adc
	clr
	am
	bnz	.checkIntersectionTrue

.checkIntersectionFalse:
	; return from the subroutine
	pi	kstack.pop
	clr						; not on an intersection, return false
	pk

	; get the intersection boundaries
.checkIntersectionTrue:
	; load the map address
	dci	maze.map

	; add y offset to DC
	clr							; 0, complemented
	com
	lr	0, A						; r0 will be our offset	
	; get the y address
	lr	A, 5
	inc							; fix -1 offset of intersections
.checkIntersectionYLoop:
	ai	%11111010					; keep subtracting 6
	bnc	.checkIntersectionYLoopEnd		; if it's < 0, then we're done
	ds	0						; "increase" r0
	br	.checkIntersectionYLoop
.checkIntersectionYLoopEnd:
	; get the true offset
	lr	A, 0
	com							; invert r0
	lr	0, A
	lr	2, A						; save Y position
	lis	12
.checkIntersectionYLoop2:
	ds	0
	bnc	.checkIntersectionYLoop2End
	adc							; add 24 bytes to the DC
	br	.checkIntersectionYLoop2		; repeat
.checkIntersectionYLoop2End:

	; add x offset to DC
;	li	$ff						; 0, complemented
	clr
	com
	lr	0, A						; r0 will be our offset	
	; get the x address
	lr	A, 4
	ai	%11111010					; subtract the initial 6
.checkIntersectionXLoop:
	ai	%11111010					; keep subtracting 6
	bnc	.checkIntersectionXLoopEnd		; if it's < 0, then we're done
	ds	0						; "increase" r0
	br	.checkIntersectionXLoop
.checkIntersectionXLoopEnd:
	; add the offset
	lr	A, 0
	com							; invert r0 to get the offset
	adc
	lr	3, A						; save X position

	; save the address
	lr	Q, DC

	; now we find this tile's boundaries	
	lm							; get the tile number
	lr	1, A						; save the tile number
	pi	checkIntersection.getBoundary		; find the boundaries
	lr	0, A						; and save those

	; find the right tile's left boundary,
	; mask it, and AND it to this one's
.checkRightBoundary:
	; check to see if we're on the edge
	lis	%1011
	xs	3
	bnz	.checkRightBoundaryTile			; not on the edge, get the boundary

	; mask off right boundary
	lis	%00001110
	ns	0
	lr	0, A
	br	.checkRightBoundaryEnd

.checkRightBoundaryTile:
	; first, fix the address
	lr	DC, Q						; retrieve the address
	lm
	; retrieve the tile address
	lm							; get the tile number
	lr	1, A						; save the tile number
	pi	checkIntersection.getBoundary		; find the boundaries
	ni	%00000010					; mask it
	sr	1
	ai	%00001110					; set the other bits
	ns	0						; and AND it with this boundary
	lr	0, A						; finally, save it
.checkRightBoundaryEnd:	

	; find the bottom tile's upper boundary,
	; mask it, and AND it to this one's
.checkBottomBoundary:
	; check to see if we're on the edge
	lis	%1001
	xs	2
	bnz	.checkBottomBoundaryTile		; not on the edge, get the boundary

	; mask off bottom boundary
	lis	%00001011
	ns	0
	lr	0, A
	br	.checkBottomBoundaryEnd

.checkBottomBoundaryTile:
	; first, fix the address
	lr	DC, Q						; retrieve the address
	lis	12
	adc
	; now, get the byte and work with it
	lm							; get the tile number
	lr	1, A						; save the tile number
	pi	checkIntersection.getBoundary		; find the boundaries
	ni	%00001000					; mask it
	sr	1
	ai	%00001011					; set the other bits
	ns	0						; and AND it with this boundary
	lr	0, A						; finally, save it	
.checkBottomBoundaryEnd:

.checkIntersectionEnd:
	; return from the subroutine
	pi	kstack.pop
	lis	1						; return true
	pk

; sub-subroutine, where we find the boundaries of a tile number

checkIntersection.getBoundary:
	; load the tile address into the DC
	dci	tiles
	lis	3						; three bytes for each tile
.checkIntersection.getBoundaryAddressLoop:
	ds	1
	bnc	.checkIntersection.getBoundaryAddressLoopEnd
	adc
	br	.checkIntersection.getBoundaryAddressLoop
.checkIntersection.getBoundaryAddressLoopEnd:
	lis	2
	adc
	lm
	; return
	pop

;---------------------------------------------------------------------------
; Check Ghost Pellet Collision
;---------------------------------------------------------------------------
; checks to see if a ghost is overlapping a pellet, and if that pellet is
; still turned on, redraws it

ghost.checkPelletCollision:
	; save return address
	lr	K, P
	pi	kstack.push

	; set the callback function
	li	>ghost.checkPelletCollision.callback
	lr	Ku, A
	li	<ghost.checkPelletCollision.callback
	lr	Kl, A
	
	; check if pacman has collided with a pellet
	jmp	pellets.checkCollision

; callback for the pellets.checkCollision routine, that redraws a pellet
; onscreen if it is still turned on, using the bit specified in r0 and the
; address at DC0

ghost.checkPelletCollision.callback:
	; save return address
	lr	K, P
	pi	kstack.push
	
	; load the plot parameters
	li	PELLET_COLOR
	lr	1, A
	lr	A, 7
	lr	2, A
	lr	A, 8
	lr	3, A
	; plot the pellet
	pi	plot
	
ghost.checkPelletCollision.callback.end:
	; retrieve the return address, store in P
	pi	kstack.pop
	lr	P, K
	
	; restore the callback location
	li	>ghost.checkPelletCollision.callback
	lr	Ku, A
	li	<ghost.checkPelletCollision.callback
	lr	Kl, A
	
	; return
	pop

;---------------------------------------------------------------------------
; Check Pacman Pellet Collision
;---------------------------------------------------------------------------
; checks to see if pac-man is overlapping a pellet, and if so, turns off its
; respective bit in RAM and increases the score

pacman.checkPelletCollision:
	; save return address
	lr	K, P
	pi	kstack.push

	; set the callback function
	li	>pacman.checkPelletCollision.callback
	lr	Ku, A
	li	<pacman.checkPelletCollision.callback
	lr	Kl, A

	; load the coordinates
	SETISAR	PACMAN_X_REG
	lr	A, I
	lr	1, A						; X position
	lr	A, S
	lr	2, A						; Y position
	
	; check if pacman has collided with a pellet
	jmp	pellets.checkCollision

; callback for the pellets.checkCollision routine, to turn off the pellet's
; respective bit in RAM. takes the bit specified in r0 and masks the bit in
; the address at DC0

pacman.checkPelletCollision.callback:
	; save return address
	lr	K, P
	pi	kstack.push
	
	; load the bitmask
	lr	A, 4
	com
	; mask the byte
	lr	DC, H
	nm
	; and store it
	lr	DC, H
	st
	
	; add to the score
	li	%01100110
	lr	Ku, A
	lr	Kl, A
	lr	Qu, A
	li	%01110110
	lr	Ql, A						; 10 points
	pi	score.add
	

	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%10000000
	bz	pacman.checkPelletCollision.callback.noextra

	lr	A, S
	inc
	sl	1
	sr	1
	lr	S, A
	
    IF switch_music
	; play the extra life sound
	dci	sfx.extraPacman
	pi	playSong

	ENDIF

	; redraw the lives display

	pi	pacman.drawLives


pacman.checkPelletCollision.callback.noextra:



	; increase dot counter
	; check for bonus fruit

	SETISAR	BONUS_PRIZE_REG
	lr	A, S
	inc
	lr	S, A
	ci	132								; fruit on if 132 dots
	bz	pacman.checkPelletCollision.callback.prize.set
	ci	54								; fruit on if 54 dots
	bz	pacman.checkPelletCollision.callback.prize.set
	br	pacman.checkPelletCollision.callback.prize.end

pacman.checkPelletCollision.callback.prize.set:
	; set prize on
	SETISAR	BONUS_PRIZE_ON
;	li	%11111111
	clr
	com
	lr	S, A
	pi	prize.draw

pacman.checkPelletCollision.callback.prize.end:


	; output dot-eating sound
	lis	$8
	sl	4
	outs	5
	clr
	outs	5

pacman.checkPelletCollision.callback.end:
	; retrieve the return address, store in P
	pi	kstack.pop
	lr	P, K
	
	li	>pacman.checkPelletCollision.callback
	lr	Ku, A
	li	<pacman.checkPelletCollision.callback
	lr	Kl, A
	pop


;---------------------------------------------------------------------------
; Pellet Check 
;---------------------------------------------------------------------------
; this function checks to see if an object at the coordinates in (r1, r2) is
; overlapping a pellet. This function should not be called directly.
;
; modifies: r0-2, r4-r8, H, K, Q, DC
; r0 = temp
; r1 = initial X position
; r2 = initial Y position
; r3 = [unused]
; r4 = bit counter
; r5 = Pac-Man's X offset
; r6 = Pac-Man's Y offset
; r7 = Pellet's X position
; r8 = Pellet's Y position

pellets.checkCollision:
	; initialize the data pointer
	dci	ram	
	; initialize the bitmask
	lis	%1000
	sl	4
	lr	4, A

	; find the X position
;	li	$fd						; -3
	lis	%00000010
	com
	lr	0, A						; store in r0 for easy subtraction
	; load the X position and adjust it
	lr	A, 1						; load Pac-Man's X position
	ai	$fb						; -5
pellets.checkCollision.XOffset:
	; save the offset
	lr	5, A
	; check if the offset is negative or 0
	ci	0
	bp	pellets.checkCollision.XOffset.end

	; shift the bitmask right
	lr	A, 4
	sr	1
	bnz	pellets.checkCollision.XOffset.noPointerAdjust
	; carry the bitmask
	lm							; increase DC0
	lis	%1000
	sl	4
	
pellets.checkCollision.XOffset.noPointerAdjust:
	lr	4, A
	; subtract 3 from the position
	lr	A, 5
	as	0
	; loop
	br	pellets.checkCollision.XOffset
pellets.checkCollision.XOffset.end:
	; find the UL pellet's coordinate
	com
	inc
	as	1
	lr	7, A

	; load the Y coordinate and adjust it
	lis	2
	as	2						; load Pac-Man's Y position
pellets.checkCollision.YOffset:
	; save the offset
	lr	6, A
	; check if the offset is negative or 0
	ci	0
	bp	pellets.checkCollision.YOffset.end

	; increment the DC0 by 3
	lis	3
	adc
	; subtract 3 from the position
	lr	A, 6
	as	0
	; loop
	br	pellets.checkCollision.YOffset
pellets.checkCollision.YOffset.end:
	; find the UL pellet's coordinate
	com
	inc
	as	2
	lr	8, A

	; preserve the DC
	lr	H, DC

pellets.checkCollision.overlap.tl:
	; check that this pellet is on
	lm
	ns	4
	bz	pellets.checkCollision.overlap.tr
	; call the callback
	pk

pellets.checkCollision.overlap.tr:
	; adjust the memory pointer
	lr	DC, H						; load the preserved DC0
	lr	A, 4
	sr	1
	bnz	pellets.checkCollision.overlap.tr.noPointerAdjust
	; overflow
	lm							; increase DC0
	lr	H, DC						; and preserve it
	lis	%1000						; roll the bitmask over
	sl	4
pellets.checkCollision.overlap.tr.noPointerAdjust:
	lr	4, A
	; adjust pellet coordinate
	lis	3
	as	7
	lr	7, A

	; check that we're overlapping this pixel
	lis	1
	as	5
	bm	pellets.checkCollision.overlap.br		; if offsetX < -1, no overlap
	; check that this pellet is on
	lm
	ns	4
	bz	pellets.checkCollision.overlap.br

	; call the callback
	pk

pellets.checkCollision.overlap.br:
	; adjust the memory pointer
	lr	DC, H						; load the preserved DC0
	lis	3						; 24 pellets, one row
	adc							; increment the pointer
	lr	H, DC						; preserve it
	; adjust pellet coordinate
	lis	3
	as	8
	lr	8, A

	; check that we're overlapping this pixel
	lis	1
	as	5
	bm	pellets.checkCollision.overlap.bl		; if offsetX < -1, no overlap
	; check that we're overlapping this pixel
	lis	1
	as	6
	bm	pellets.checkCollision.end			; if offsetY < -1, no overlap
	; check that this pellet is on
	lm
	ns	4
	bz	pellets.checkCollision.overlap.bl

	; call the callback
	pk

pellets.checkCollision.overlap.bl:
	; adjust the memory pointer
	lr	DC, H						; load the preserved DC0
	lr	A, 4
	sl	1
	bnz	pellets.checkCollision.overlap.bl.noPointerAdjust
	; overflow
	ds	11						; decrease the stored DC0
	lr	DC, H						; and load it
	lis	%0001						; roll the bitmask over
pellets.checkCollision.overlap.bl.noPointerAdjust:
	lr	4, A
	; adjust pellet coordinate
	ds	7
	ds	7
	ds	7

	; check that we're overlapping this pixel
	lis	1
	as	6
	bm	pellets.checkCollision.end			; if offsetY < -1, no overlap
	; check that this pellet is on
	lm
	ns	4
	bz	pellets.checkCollision.end

	; call the callback
	pk
	
pellets.checkCollision.end:
	; return
	pi	kstack.pop
	pk

;---------------------------------------------------------------------------
; Powerpellet Check 
;---------------------------------------------------------------------------
; this function checks to see if an object at the coordinates in (r1, r2) is
; overlapping a powerpellet. this function should not be called directly.
;
; modifies: r1-r6
; r1 = Pellet's X position
; r2 = Pellet's Y position, X coordinate for sprite.draw
; r3 = store Y coordinate for sprite.draw
; r4 = initial X position
; r5 = initial Y position
; r6 = powerpellets reg copy

powerPellets.checkCollision:
	; save return address
	lr	K, P
	pi	kstack.push

	; copy the registers
	SETISAR	POWERPELLETS_REG
	lr	A, S
	lr	6, A
	lr	A, 1
	lr	4, A
	lr	A, 2
	lr	5, A
	
powerPellets.checkCollision.prize:
	; start checking if we have hit prize
	
	SETISAR	BONUS_PRIZE_ON
	clr
	as	S	
	bz	powerPellets.checkCollision.prize.jump		; check powerpellets instead

	; prize is on, check if it's hit
	; y coordinate
	lr	A, 5
	ci	29
	bnz	powerPellets.checkCollision.prize.jump		; not on correct y coordinate, no hit
	lr	A, 4
	ci	34								; left "hitpoint"
	bp	powerPellets.checkCollision.prize.jump		; if positive, not hit
	ci	43								; right hitpoint
	bm	powerPellets.checkCollision.prize.jump		; if negative, not hit


	; prize is hit, redraw if ghost has hit, erase and add score if it's pacman

	lr	A, 6
	ni	%00010000						; check if pacman has collided
	bz	powerPellets.checkCollision.prize.draw	; ghost has collided, branch

	; check if pac-man is centered on prize, if not redraw
	lr	A, 4
	ci	39
	bnz	powerPellets.checkCollision.prize.draw


	; pacman has hit prize

	IF switch_music
	; play prize-eating sound
	dci	sfx.prizeEat
	pi	playSong
	ENDIF

	SETISAR	BONUS_PRIZE_ON
	clr	
	lr	S, A							; clear bonus prize


	; branch out of range:
	br	powerPellets.checkCollision.prize.score.add
powerPellets.checkCollision.prize.jump:
	br	powerPellets.checkCollision.pacman.x1


powerPellets.checkCollision.prize.score.add:
	;add score
	; level 1: 100; 2: 300; 3,4: 500; 5,6: 700; 
	; level 7,8: 1000; 9,10: 2000; 11,12: 3000; >12: 5000


	; changed below from v b29b


	; check level to set correct points

	dci	prize.score.table
	SETISAR	LEVEL_NUMBER_REG
	lr	A, S
	ni	%10000000
	bnz	prize.5000
	lr	A, S
	ci	12
	bm	prize.5000
	adc	
prize.5000:
	lm
	br	powerPellets.checkCollision.prize.score


prize.score.table:
	.byte	%10110110					; 50
	.byte	%01100111					; 01
	.byte	%01101001					; 03
	.byte	%01101011					; 05
	.byte	%01101011					; 05
	.byte	%01101100					; 07
	.byte	%01101100					; 07
	.byte	%01110110					; 10
	.byte	%01110110					; 10
	.byte	%10000110					; 20
	.byte	%10000110					; 20
	.byte	%10010110					; 30
	.byte	%10010110					; 30




	; changed above from v b29b



powerPellets.checkCollision.prize.score:

	lr	4, A
	
	; add to the score
	li	%01100110					; 00
	lr	Ku, A
	lr	Kl, A
	lr	A, 4
	lr	Qu, A						; 01, 03, 05, 07, 10, 20, 30 or 50
	li	%01100110					; 00
	lr	Ql, A
	pi	score.add

	; check if we've hit extralife
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%10000000
	bz	powerPellets.checkCollision.prize.draw	; no extra life, redraw/erase

	lr	A, S					; add life, remove extralife-marker
	inc
	sl	1
	sr	1
	lr	S, A
	
	IF switch_music
	; play the extra life sound
	dci	sfx.extraPacman
	pi	playSong
	ENDIF

	; redraw the lives display
	pi	pacman.drawLives

powerPellets.checkCollision.prize.draw:

	; draw/erase prize
	pi	prize.draw

	jmp	powerPellets.checkCollision.end	; end check


	; check coordinates
powerPellets.checkCollision.pacman.x1:
	; check that the x value is in range in left column
	lr	A, 4
	ci	2
	bp	powerPellets.checkCollision.pacman.x2
	ci	9
	bm	powerPellets.checkCollision.pacman.x2
	; load the pellet's coordinate
	lis	7
	lr	2, A
	lr	A, 6
	ni	%11110110
	lr	6, A
	br	powerPellets.checkCollision.pacman.y1
powerPellets.checkCollision.pacman.x2
	; check that the x value is in range in right column
	lr	A, 4
	ci	68
	bp	powerPellets.checkCollision.pacman.jump.to.end
	ci	75
	bm	powerPellets.checkCollision.pacman.jump.to.end
	; load the pellet's coordinate
	li	73
	lr	2, A
	lr	A, 6
	ni	%11111001
	lr	6, A
	br	powerPellets.checkCollision.pacman.y1
	
powerPellets.checkCollision.pacman.y1:
	; check that the y value is in range in top row
	lr	A, 5
	ci	%11111110					; -2
	bp	powerPellets.checkCollision.pacman.y2
	ci	5
	bm	powerPellets.checkCollision.pacman.y2
	; load the pellet's coordinate
	lis	3
	lr	3, A
	lr	A, 6
	oi	%000000011
	lr	6, A
	br	powerPellets.checkCollision.pacman.collided
powerPellets.checkCollision.pacman.y2
	; check that the y value is in range in bottom row
	lr	A, 5
	ci	37
	bp	powerPellets.checkCollision.end
	ci	44
	bm	powerPellets.checkCollision.end
	; load the pellet's coordinate
	li	42
	lr	3, A
	lr	A, 6
	oi	%000001100
	lr	6, A
	br powerPellets.checkCollision.pacman.collided
	
powerPellets.checkCollision.pacman.jump.to.end:
    jmp powerPellets.checkCollision.pacman.end
    


powerPellets.checkCollision.end:
	; return
	pi	kstack.pop
	pk



powerPellets.checkCollision.pacman.collided:

	lr	A, 6
	ni	%00010000						; check if pacman has collided
	bz	powerPellets.checkCollision.ghost		; ghost has collided, branch

	; p.p. isn't eaten unless pac-man is on top of it

	lr	A, 2
	com
	inc
	as	4
	inc
	bnz	powerPellets.checkCollision.end
	lr	A, 3
	com
	inc
	as	5
	inc
	bnz	powerPellets.checkCollision.end


	; pacman has has collided with a powerpellet, erase it
	; r6 is xxxx1110, xxxx1101, xxxx1011 or xxxx0111

	SETISAR	POWERPELLETS_REG				
	lr	A, 6
	ns	S							; AND with original to wipe out P.P
	lr	6, A							; store the new pattern in r6
	xs	S							; compare result to original
	bz	powerPellets.checkCollision.end		; end if there was no change

	; a powerpellet has been eaten
	lr	A, 6
	oi	%00100000						; set bit 5 - ghost slow down
	lr	S, A							; store new power pellet register

	; set all ghosts to eatable
	SETISAR	GHOST_EATABLE_REG
	lr	A, S
	oi	%00001111
	lr	S, A


	; set all ghosts to first time blue (to change direction)
	lisu	4
	lisl	2
	
	lr	 A, S
	oi	%00100000
	lr 	S, A

	lisl	6

	lr	 A, S
	oi	%00100000
	lr 	S, A

	lisu	5

	lr	 A, S
	oi	%00100000
	lr 	S, A

	lisl	2

	lr	 A, S
	oi	%00100000
	lr 	S, A


	; erase the power pellet from the maze
	li	MAZE_BGCOLOR
	lr	1, A
	; load the sprite number
	li	23
	lr	4, A	
	pi	sprite.draw	
	
	; add to the score
	li	%01100110
	lr	Ku, A
	lr	Kl, A
	lr	Qu, A
	li	%10110110
	lr	Ql, A						; 50 points
	pi	score.add

	; check if we've hit extralife
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%10000000
	bz	powerPellets.checkCollision.pacman.reset.ghost.timer

	; increase lives
	lr	A, S
	inc
	sl	1
	sr	1
	lr	S, A
	
	IF switch_music
	; play the extra life sound
	dci	sfx.extraPacman
	pi	playSong
	ENDIF

	; redraw the lives display
	pi	pacman.drawLives

powerPellets.checkCollision.pacman.reset.ghost.timer:
	pi	set.slow.timer

powerPellets.checkCollision.pacman.end:
	br	powerPellets.checkCollision.end


powerPellets.checkCollision.ghost:

	; ghost has has collided with a powerpellet, redraw it
	; r6 is xxxx1110, xxxx1101, xxxx1011 or xxxx0111

	SETISAR	POWERPELLETS_REG				
	lr	A, 6
	ns	S							; AND with original to wipe out P.P
	xs	S							; compare result to original
	bz	powerPellets.checkCollision.end		; end if there was no change

	li	POWERPELLET_COLOR
	lr	1, A
	; load the sprite number
	li	23
	lr	4, A	
	; draw the pellet
	pi	sprite.draw	

	br	powerPellets.checkCollision.end	

;===========================================================================
; Score Functions 
;===========================================================================
;---------------------------------------------------------------------------
; Add score 
;---------------------------------------------------------------------------

; adds a 4-byte decimal adjusted number to the score
; stored in r12-r15 (K, Q)

; SCORE_REG is 'O'24-'0'27


	
score.add:

	; First check if extra-life is possible:
	SETISAR	[SCORE_REG + 1]
	clr
	as	D	
	bnz	score.add.no.extralife		; not 00 on byte 2
	clr
	as	S
	bnz	score.add.no.extralife		; not 00 on byte 1

	
score.add.extralife.perhaps:
	lisl	7
	lr	A, Ql		; load least significant 
	asd	S		; decimal add scratchpad (score_reg)
	lr	D, A		; save result and then decrease sp
	lr	A, Qu		; load next byte into A
	lnk			; add carry from previous operation
	asd	S		; decimal add scratchpad (byte 2)
	lr	D, A		; store result
	lr	A, Kl		; load next byte
	lnk			; add carry from previous operation
	asd	S		; decimal add scratchpad (byte 3)
	lr	D, A		; store result
	lr	A, Ku		; load next byte
	lnk			; add carry from previous operation
	asd	S		; decimal add scratchpad
	lr	I, A		; store result, increase ISAR to check 
				; for extralife
	lr	A, S		; as we had score 0000xxxx check low nibble
	sl	4		; of byte 2, maximum score increase is 5000
	sr	4		; if we had 00009990 we can only get maximum 00014990
	ci	1			; is low nibble ONE?
	bnz	score.add.end	; not 1 means not yet >= 10000...

	; total machine cycles from *.perhaps: 25
	
	; extra life
	
	; set marker for extra life
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	oi	%10000000
	lr	S, A

	br	score.add.end

score.add.no.extralife:
	lisl	7
	lr	A, Ql				; load least significant 
	asd	S				; decimal add scratchpad (score_reg)
	lr	D, A				; save result and then decrease scratchpad
	lr	A, Qu				; load next byte into A
	lnk					; add carry from previous operation
	asd	S				; decimal add scratchpad (byte 2)
	lr	D, A				; store result
	lr	A, Kl				; load next byte
	lnk					; add carry from previous operation
	asd	S				; decimal add scratchpad (byte 3)
	lr	D, A				; store result
	lr	A, Ku				; load next byte
	lnk					; add carry from previous operation
	asd	S				; decimal add scratchpad
	lr	D, A				; store result 

	
						; done						

score.add.end:
	; return from the subroutine
	pop

;===========================================================================
; Drawing Functions 
;===========================================================================

	; basic drawing functions
	include "drawing.inc"

;---------------------------------------------------------------------------
; Draw Tile 
;---------------------------------------------------------------------------
; draw an 8x8 tile from a data pointer
; r1 = color 1 (off)
; r2 = color 2 (on)
; r3 = x position
; r4 = y position
; r5 = tile number
;
; modifies: r1-r9 (blit), Q, DC

drawTile:
	; get the tile address
	dci	tiles
	; add the offset
	lr	A, 5
	inc							; make sure we hit 0
	lr	0, A
	lis	3						; three bytes for each tile
.drawTileAddressLoop:
	ds	0
	bz	.drawTileAddressLoopEnd
	adc
	br	.drawTileAddressLoop
.drawTileAddressLoopEnd:
	; got the tile data, now get the graphics address
	lm
	lr	Qu, A
	lm
	lr	Ql, A
	lr	DC, Q

	; load the width and height
	lis	6
	lr	5, A
	lr	6, A

.drawTileBlit:
	; draw the tile itself
	jmp	blit

;---------------------------------------------------------------------------
; Draw Map 
;---------------------------------------------------------------------------
; draw the pac-man maze in two colors
; r0 = loop counter
; r1-r9 = routine arguments
; r10 = x counter
; r11 = y counter

; r1 needs to be set before calling this routine

map.draw:
	; load the return address into memory
	lr	K, P
	pi	kstack.push

	; load the map address into DC1
	dci	maze.map
	xdc
	
	; load the initial X and Y positions of the tiles
	lis	5
	lr	3, A
;	li	$fe						; -2
	lis	1
	com
	lr	4, A

	; load r10 with 10 rows of tiles
	lis	10
	lr	10, A
map.draw.loop.row:

	; load r11 with 12 columns of tiles
	lis	12
	lr	11, A
map.draw.loop.column:

	; note of reference to drawTile:
	; r1 = color 1
	; r2 = color 2
	; r3 = x (to screen)
	; r4 = y (to screen)
	; r5 = tile number

map.draw.loop.drawTile:

	; this block of code gets the next tile number
	; and increases the data offset

	; swap DC0 and DC1
	xdc
	; get and save the next tile byte
	lm
	lr	5, A
	; swap back
	xdc

	; set the colors
	; color 1
;	li	MAPDRAW_BGCOLOR
;	lr	1, A						; r1
	; color 2
	li	MAZE_FGCOLOR
	lr	2, A						; r2

	; finally, call the tile function
	pi	drawTile
	
	; drawTile adds 4 to both the X and Y positions

map.draw.loop.column.end:
	; increase the tile X position
	lis	2
	as	3
	lr	3, A
	; reset the tile Y position
;	li	$fc
	lis	%0011
	com
	as	4
	lr	4, A
	; decrement column counter
	ds	11
	bnz	map.draw.loop.column
	
map.draw.loop.row.end:
	; reset the tile X position
	lis	5
	lr	3, A
	; increase the tile Y position
	lis	6
	as	4
	lr	4, A
	; decrement row counter
	ds	10
	bnz	map.draw.loop.row

map.draw.drawGhostGate:
	; draw the exit gate for the ghost
	li	GATE_COLOR
	lr	1,A
	li	39
	lr	2,A
	li	22
	lr	3,A
	li	19
	lr	4,A
	li	%11111000
	lr	5,A
	pi	sprite.draw	
	
map.draw.end:
	; return to the program flow
	pi	kstack.pop
	pk							; return from the subroutine

;---------------------------------------------------------------------------
; Draw Sprite 
;---------------------------------------------------------------------------
; draw a 8x5 sprite from a data pointer
; r1 = color
; r2 = x (to screen)
; r3 = y (to screen)
; r4 = sprite number

sprite.draw:
	; blit reference:
	; r1 = color 1 (off)
	; r2 = color 2 (on)
	; r3 = x position
	; r4 = y position
	; r5 = width
	; r6 = height

	; get the tile address
	dci	sprites
	; add the offset
	lr	A, 4
	inc							; make sure we hit 0
	lr	0, A
	lis	2						; two bytes for each sprite
sprite.draw.addressLoop:
	ds	0
	bz	sprite.draw.addressLoop.end
	adc
	br	sprite.draw.addressLoop
sprite.draw.addressLoop.end:
	; load the address for this sprite's number
	lm
	lr	Qu, A
	lm
	lr	Ql, A
	lr	DC, Q
	
	; load the sprite size (8x5)
	lis	8
	lr	5, A
	lis	5
	lr	6, A
	; load the position
	lr	A, 3
	lr	4, A
	lr	A, 2
	lr	3, A
	; load the colors
	lr	A, 1
	lr	2, A
	li	clear
	lr	1, A

	; draw the sprite
	jmp	blit
	
;---------------------------------------------------------------------------
; Draw Sidebar 
;---------------------------------------------------------------------------

; draws the right sidebar

sidebar.draw:
	; save return address
	lr	K, P
	pi	kstack.push

	; set sidebar color
	li	SIDEBAR_COLOR
	lr	1, A

	; start row loop
	li	57
	lr	3, A
sidebar.draw.row:
	li	102			; changed from b29b, var 97
	lr	2, A
sidebar.draw.column:
	pi	plot

sidebar.drawCheck:
	; check column
	ds	2
	lr	A, 2
	ci	76						; see if we've reached the left edge
	bnz	sidebar.draw.column

	; check row
	ds	3
	bc	sidebar.draw.row					; branch if > 0

sidebar.drawText:
	; draw "SCORE"
	dci	gfx.score.parameters
	pi	blitGraphic
	; draw "HIGH"
	dci	gfx.high.parameters
	pi	blitGraphic

sidebar.drawEnd:
	; return from subroutine
	pi	kstack.pop
	pk

;---------------------------------------------------------------------------
; Draw Hi-Score 
;---------------------------------------------------------------------------
; draws the 8-digit hi-score to the screen

hiscore.draw:
	; save return address
	lr	K, P

	; load colors
	li	SIDEBAR_COLOR
	lr	1, A
	li	SCORE_COLOR
	lr	2, A

	; load position
	li	78
	lr	10, A
	li	24
	lr	11, A

	; set the number counter (to detect when to wrap)
	lis	8
	lr	0, A
	; set ISAR

	; fetch hi-score from RAM  
	; and store in registers
	dci	ram + $602
	SETISAR	HISCORE_REG
	lm
	lr	I, A
	lm
	lr	I, A
	lm
	lr	I, A
	lm
	lr	S, A
	SETISAR HISCORE_REG
	; branch to draw score
	br	score.draw.loop

;---------------------------------------------------------------------------
; Draw Score 
;---------------------------------------------------------------------------
; draws the 8-digit score to the screen

score.draw:
	; save return address
	lr	K, P

	; load colors
	li	SIDEBAR_COLOR
	lr	1, A
	li	SCORE_COLOR
	lr	2, A

	; load position
	li	78
	lr	10, A
	lis	6
	lr	11, A

	; set the number counter (to detect when to wrap)
	lis	8
	lr	0, A
	; set ISAR
	SETISAR SCORE_REG

score.draw.loop:
	; set DC
	dci	gfx.numbers

	; load size
	lis	3
	lr	5, A
	lis	5
	lr	6, A

	; load position
	lr	A, 10
	lr	3, A
	lr	A, 11
	lr	4, A

	; see if the tile is even or odd
	lis	%1
	ns	0
	bz	score.draw.evenByte				; if even, branch

score.draw.oddByte:
	; get lower half-byte
	lis	%1111
	ns	I						; set ISAR to next byte
	br	score.draw.drawDigit
score.draw.evenByte:
	; get upper half-byte
	lr	A, S
	sr	4

score.draw.drawDigit:
	; add tile offset
	sl	1
	adc
	; and draw the number
	pi	blit

	; increase x position
	lis	4
	as	10
	lr	10, A						; each tile is three pixels + one pixel space

	; decrease the counter
	ds	0
	bz	score.draw.end					; reached the end, break
	; check to see if we're on tile 4
	lis	4
	xs	0
	bnz	score.draw.loop

score.draw.changeRow:
	; increase X position
	lis	%1011
	com
	as	10
	lr	10, A
	; increase Y position
	lis	6
	as	11
	lr	11, A
	br	score.draw.loop

score.draw.end:
	; return from subroutine
	pk

;===========================================================================
; Object Drawing Functions 
;===========================================================================

;---------------------------------------------------------------------------
; Draw Pellets 
;---------------------------------------------------------------------------
; this function draws out the pellets as specified by "maze.pellets" and
; "maze_powerPellets"

pellets.draw:
	; save return address
	lr	K, P

	; load our data pointer
	dci	ram

	; load our color
	li	PELLET_COLOR
	lr	1, A

	; set the initial coordinates
	lis	5						; 5
	lr	2, A						; x coordinate
	lis	1						; -2
	com
	lr	3, A						; y coordinate

	; load our byte loop counter
	li	60
	lr	4, A
pellets.draw.byteLoop:

	; load next byte
	lm
	lr	6, A

	; load our bit mask
	lis	%1000
	sl	4
	lr	5, A
pellets.draw.bitLoop:
	; see if there's a pellet here
	lr	A, 6
	ns	5
	; if not, branch
	bz	pellets.draw.bitLoopEnd

	; plot the pixel
	pi	plot
	
pellets.draw.bitLoopEnd:
	; add three to the x coordinate
	lr	A, 2
	ai	3						; add three pixels
	lr	2, A						; store it back in
	; if we've reach the end, roll over
	ci	77
	bnz	pellets.draw.loopCheck

	; reset x, add 3 to y
	lis	5
	lr	2, A
	lr	A, 3
	ai	3						; add three pixels
	lr	3, A						; store it back in the y coordinate

pellets.draw.loopCheck:
	; check bit loop
	lr	A, 5
	sr	1
	lr	5, A
	bnz	pellets.draw.bitLoop

	; check byte loop
	ds	4
	bnz	pellets.draw.byteLoop

pellets.draw.end:
	; return from the subroutine
	pk


;---------------------------------------------------------------------------
; Draw Prize
;---------------------------------------------------------------------------
; uses r10 and r11 for x and y coordinates (sidebar plot)

prize.draw:
	; save return address
	lr	K, P

	; check if prize is on, if not, erase it from board
	SETISAR	BONUS_PRIZE_ON
	li	39						; x-coordinate	
	lr	10, A						; store in r10
	li	29						; y-coordinate
	lr	11, A						; store in r11
	clr
	as	S						; Scratchpad + A -> A - bonus prize ON?
	bnz	prize.draw.set				; coordinates for gamefield plot loaded

prize.draw.erase:
	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	li	39
	lr	2, A
	; load y coordinate
	li	29
	lr	3, A
	; load the sprite number
	clr
	lr	4, A
	; erase the prize
	pi	sprite.draw

	; draw prize

prize.draw.set:

	SETISAR	LEVEL_NUMBER_REG
	lr	A, S
	ni	%10000000
	bnz	prize.draw.set.5000
	lr	A, S
	ci	1
	bz	prize.draw.set.100
	ci	2
	bz	prize.draw.set.300.jump
	ci	4
	bt	3,prize.draw.set.500.jump
	ci	6
	bt	3,prize.draw.set.700.jump
	ci	8
	bt	3,prize.draw.set.1000.jump
	ci	10
	bt	3,prize.draw.set.2000.jump
	ci	12
	bt	3,prize.draw.set.3000.jump

	; key for >12
	br	prize.draw.set.5000

prize.draw.set.300.jump:
	jmp	prize.draw.set.300
prize.draw.set.500.jump:
	jmp	prize.draw.set.500
prize.draw.set.700.jump:
	jmp	prize.draw.set.700
prize.draw.set.1000.jump:
	jmp	prize.draw.set.1000
prize.draw.set.2000.jump:
	jmp	prize.draw.set.2000
prize.draw.set.3000.jump:
	jmp	prize.draw.set.3000

prize.draw.set.5000:

	; key

	; check if prize is on, if not, erase it
	SETISAR	BONUS_PRIZE_ON
	clr
	as	S
	bnz	prize.draw.set.5000.game

	li	85
	lr	10, A
	li	38
	lr	11, A

	; clear bkg
	li	SIDEBAR_COLOR
	lr	1, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; load the sprite number
	clr
	lr	4, A
	; draw the prize
	pi	sprite.draw

	; clear bkg
	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; load the sprite number
	li	52
	lr	4, A
	; draw the prize
	pi	sprite.draw	


prize.draw.set.5000.game:
	lis	green
	lr	1, A
	; load the sprite number
	li	46
	lr	4, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; draw the prize
	pi	sprite.draw

	li	blue
	lr	1, A
	; load the sprite number
	li	47
	lr	4, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; draw the prize
	pi	sprite.draw

	jmp	prize.draw.end


prize.draw.set.100:

	; cherries

	; check if prize is on, if not, erase it
	SETISAR	BONUS_PRIZE_ON
	lr	A, S
	ci	0
	bnz	prize.draw.set.100.game

	li	78
	lr	10, A
	li	38
	lr	11, A
	

prize.draw.set.100.game:
	lis	green
	lr	1, A
	; load the sprite number
;	li	32
	lis	%0010
	sl	4
	lr	4, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; draw the prize
	pi	sprite.draw

	li	red
	lr	1, A
	; load the sprite number
	li	33
	lr	4, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; draw the prize
	pi	sprite.draw

	jmp	prize.draw.end

prize.draw.set.300:

	; strawberry

	; check if prize is on, if not, erase it
	SETISAR	BONUS_PRIZE_ON
	lr	A, S
	ci	0
	bnz	prize.draw.set.300.game

	li	85
	lr	10, A
	li	38
	lr	11, A

	; clear bkg
	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; load the sprite number
;	li	48
	lis	%0011
	sl	4
	lr	4, A
	; draw the prize
	pi	sprite.draw	

prize.draw.set.300.game:
	lis	green
	lr	1, A
	; load the sprite number
	li	34
	lr	4, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; draw the prize
	pi	sprite.draw

	li	red
	lr	1, A
	; load the sprite number
	li	35
	lr	4, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; draw the prize
	pi	sprite.draw

	jmp	prize.draw.end

prize.draw.set.500:

	; orange

	; check if prize is on, if not, erase it
	SETISAR	BONUS_PRIZE_ON
	clr
	as	S
	bnz	prize.draw.set.500.game

	li	92
	lr	10, A
	li	38
	lr	11, A

	; clear bkg
	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; load the sprite number
	li	49
	lr	4, A
	; draw the prize
	pi	sprite.draw	
	

prize.draw.set.500.game:
	lis	green
	lr	1, A
	; load the sprite number
	li	36
	lr	4, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; draw the prize
	pi	sprite.draw

	li	red
	lr	1, A
	; load the sprite number
	li	37
	lr	4, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; draw the prize
	pi	sprite.draw

	jmp	prize.draw.end

prize.draw.set.700:


	; apple

	; check if prize is on, if not, erase it
	SETISAR	BONUS_PRIZE_ON
	lr	A, S
	ci	0
	bnz	prize.draw.set.700.game

	li	78
	lr	10, A
	li	45
	lr	11, A
	
	; clear bkg
	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; load the sprite number
	li	50
	lr	4, A
	; draw the prize
	pi	sprite.draw	


prize.draw.set.700.game:
	lis	green
	lr	1, A
	; load the sprite number
	li	38
	lr	4, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; draw the prize
	pi	sprite.draw

	li	red
	lr	1, A
	; load the sprite number
	li	39
	lr	4, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; draw the prize
	pi	sprite.draw

	jmp	prize.draw.end

prize.draw.set.1000:

	; grenade

	; check if prize is on, if not, erase it
	SETISAR	BONUS_PRIZE_ON
	lr	A, S
	ci	0
	bnz	prize.draw.set.1000.game

	li	85
	lr	10, A
	li	45
	lr	11, A

prize.draw.set.1000.game:
	lis	green
	lr	1, A
	; load the sprite number
	li	40
	lr	4, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; draw the prize
	pi	sprite.draw

	jmp	prize.draw.end

prize.draw.set.2000:

	; spaceship

	; check if prize is on, if not, erase it
	SETISAR	BONUS_PRIZE_ON
	lr	A, S
	ci	0
	bnz	prize.draw.set.2000.game

	li	92
	lr	10, A
	li	45
	lr	11, A

	
	; clear bkg
	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; load the sprite number
	li	51
	lr	4, A
	; draw the prize
	pi	sprite.draw	

	

prize.draw.set.2000.game:
	lis	green
	lr	1, A
	; load the sprite number
	li	41
	lr	4, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; draw the prize
	pi	sprite.draw

	li	blue
	lr	1, A
	; load the sprite number
	li	42
	lr	4, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; draw the prize
	pi	sprite.draw

	li	red
	lr	1, A
	; load the sprite number
	li	43
	lr	4, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; draw the prize
	pi	sprite.draw

	jmp	prize.draw.end

prize.draw.set.3000:

	; bell

	; check if prize is on, if not, erase it
	SETISAR	BONUS_PRIZE_ON
	lr	A, S
	ci	0
	bnz	prize.draw.set.3000.game

	li	78
	lr	10, A
	li	38
	lr	11, A

	; clear bkg
	li	blue
	lr	1, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; load the sprite number
	clr
	lr	4, A
	; draw the prize
	pi	sprite.draw	

prize.draw.set.3000.game:
	li	red
	lr	1, A
	; load the sprite number
	li	44
	lr	4, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; draw the prize
	pi	sprite.draw

	li	blue
	lr	1, A
	; load the sprite number
	li	45
	lr	4, A
	; load x coordinate
	lr	A, 10
	lr	2, A
	; load y coordinate
	lr	A, 11
	lr	3, A
	; draw the prize
	pi	sprite.draw

prize.draw.end:
	; return from the subroutine
	pk

;---------------------------------------------------------------------------
; Draw Power Pellets 
;---------------------------------------------------------------------------
; this function draws out the power pellets 

; 1:(7,3)  
; 2:(73,3)
; 3:(73,42)
; 4:(7,42)

powerPellets.draw:
	; save return address
	lr	K, P

powerPellets.draw.0:
	; check if this pellet is on
	SETISAR	POWERPELLETS_REG
	lr	A, S
	ni	%00001000
	bz	powerPellets.draw.1

	; load the color
	li	POWERPELLET_COLOR
	lr	1, A
	SETISAR	GLOBAL_TIMER_REG
	lr	A, S
	ni	%00000010
	bz	powerPellets.draw.0.ppcol
	li	bkg
	lr	1, A
powerPellets.draw.0.ppcol:
	; load x coordinate
	lis	7
	lr	2, A
	; load y coordinate
	lis	3
	lr	3, A
	; load the sprite number
	li	23
	lr	4, A
	
	; draw the pellet
	pi	sprite.draw
	
powerPellets.draw.1:
	; check if this pellet is on
	SETISAR	POWERPELLETS_REG
	lr	A, S
	ni	%00000100
	bz	powerPellets.draw.2

	; load the color
	li	POWERPELLET_COLOR
	lr	1, A
	SETISAR	GLOBAL_TIMER_REG
	lr	A, S
	ni	%00000010
	bz	powerPellets.draw.1.ppcol
	li	bkg
	lr	1, A
powerPellets.draw.1.ppcol:
	; load x coordinate
	li	73
	lr	2, A
	; load y coordinate
	lis	3
	lr	3, A
	; load the sprite number
	li	23
	lr	4, A
	
	; draw the pellet
	pi	sprite.draw

powerPellets.draw.2:
	; check if this pellet is on
	SETISAR	POWERPELLETS_REG
	lr	A, S
	ni	%00000010
	bz	powerPellets.draw.3

	; load the color
	li	POWERPELLET_COLOR
	lr	1, A
	SETISAR	GLOBAL_TIMER_REG
	lr	A, S
	ni	%00000010
	bz	powerPellets.draw.2.ppcol
	li	bkg
	lr	1, A
powerPellets.draw.2.ppcol:
	; load x coordinate
	li	73
	lr	2, A
	; load y coordinate
	li	42
	lr	3, A
	; load the sprite number
	li	23
	lr	4, A
	
	; draw the pellet
	pi	sprite.draw

powerPellets.draw.3:
	; check if this pellet is on
	SETISAR	POWERPELLETS_REG
	lr	1, A
	lr	A, S
	ni	%00000001
	bz	powerPellets.draw.end

	; load the color
	li	POWERPELLET_COLOR
	lr	1, A
	SETISAR	GLOBAL_TIMER_REG
	lr	A, S
	ni	%00000010
	bz	powerPellets.draw.3.ppcol
	li	bkg
	lr	1, A
powerPellets.draw.3.ppcol:
	; load x coordinate
	lis	7
	lr	2, A
	; load y coordinate
	li	42
	lr	3, A
	; load the sprite number
	li	23
	lr	4, A
	
	; draw the pellet
	pi	sprite.draw


powerPellets.draw.end:
	; return from the subroutine
	pk

;---------------------------------------------------------------------------
; Clear Ghosts 
;---------------------------------------------------------------------------


ghosts.clear:
	; save return address
	lr	K, P
	pi	kstack.push



	; clear the ghosts
	SETISAR GHOST_NUMBER_REG
	clr
	lr	S, A

ghosts.clear.loop:
	; move registers to temp location
	; get ISAR location for ghost
	SETISAR GHOST_NUMBER_REG
	; multiply by 4 bytes per ghost
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	IS, A
	lr	10, A

ghosts.clear.loop.erase:
	; clear the ghost
	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, D
	lr	3, A
	; set sprite index
	clr
	lr	4, A
	; set bitmask
	lis	%0111
	com
	lr	5, A
	; draw the sprite
	pi	sprite.draw

ghosts.clear.loop.redrawPellets:
	; restore ISAR
	lr	A, 10
	lr	IS, A
	
	; redraw overlapped pellets
	lr	A, I
	lr	1, A
	lr	A, D
	lr	2, A
	pi	ghost.checkPelletCollision

ghosts.clear.loop.checkLoop:
	; check if we've reached the last ghost
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	inc
	ci	GHOST_NUMBER
	bz	ghosts.clear.end
	; update next ghost
	lr	S, A
	br	ghosts.clear.loop
ghosts.clear.end:
	; return from the subroutine
	pi	kstack.pop
	pk



;===========================================================================
; Pac-Man Functions 
;===========================================================================

;---------------------------------------------------------------------------
; Clear Pac-Man 
;---------------------------------------------------------------------------
; erase Pac-Man to the background color

pacman.clear:
	; load the maze's bgcolor
	li	MAZE_BGCOLOR
	lr	1, A
	SETISAR	PACMAN_X_REG
	; load x coordinate
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	; load sprite number (0, clear)
	clr
	lr	4, A
	; draw the sprite
	jmp	sprite.draw

;---------------------------------------------------------------------------
; Draw Pac-Man 
;---------------------------------------------------------------------------
; draws pac-man based on the data in all the pac-man registers

pacman.draw:
	; save return address
	lr	K, P

	; load Pac-Man's color
	li	PACMAN_COLOR
	lr	1, A

	; If pac-man is dead, jump directly to draw
	li	%10101010
	xs	7
	bnz	pacman.draw.init
	jmp	pacman.draw.sprite

pacman.draw.init:
	; load Pac-Man's direction
	SETISAR	PACMAN_DIRECTION_REG
	lr	A, S						; r37

pacman.draw.isDirectionDown:
	ci	%00000100					; is it down?
	bm	pacman.draw.directionUp			; negative result means up
	bnz	pacman.draw.isDirectionRight		; if not zero check if right
	lis	5						; set index to #5
	br	pacman.draw.directionEnd

pacman.draw.isDirectionRight:
	ci	%00000001					; is it right?
	bnz	pacman.draw.directionLeft		; if not direction is left
	lis	13						; set index to #13
	br	pacman.draw.directionEnd   

pacman.draw.directionUp:
	lis	1						; set index to #1
	br	pacman.draw.directionEnd

pacman.draw.directionLeft:
	lis	9						; set index to #9

pacman.draw.directionEnd:
	lr	4, A						; store index in r4

	; find the sprite's animation offset
	SETISAR	PACMAN_ANIMINDEX_REG
	lr	A, S						; load the animation index
	lr	0, A						; r0 is our loop counter
	lr	A, 4						; get back the original index
pacman.draw.getOffset:
	ds	0
	bm	pacman.draw.getOffsetEnd
	inc							; use the next tile
	br	pacman.draw.getOffset
pacman.draw.getOffsetEnd:
	lr	4, A						; set the index again

pacman.draw.warp:
	; set preliminary bitmask for when we're not looping
	lis	%0111
	com
	lr	5, A

	SETISAR	PACMAN_X_REG
	lr	A, S
	ci	128						; check if it's off the left side
	bp	pacman.draw.warpCheckRight			; if not, check the right

	; now we find the bitmask for pacman
	com							; invert the position and increase it
	inc							; to get the number of bits to be masked
	lr	0, A						; save it
	lis	%0111						; load our new bitmask
	com
pacman.draw.warpLeftLoop:
	ds	0
	bm	pacman.draw.warpLeftLoopEnd			; if we've reached the end of the bits, end the loop
	sr	1
	br	pacman.draw.warpLeftLoop
pacman.draw.warpLeftLoopEnd:
	lr	5, A						; save the bitmask
	br	pacman.draw.warpEnd				; end the warping check

pacman.draw.warpCheckRight:
	lr	A, S
	ci	75						; check if it's off the right side
	bp	pacman.draw.warpEnd				; if not, end the loop

	; now we find the bitmask for pacman
	ai	%10110101					; subtract 75 pixels to get the number of bits to be masked
	lr	0, A						; save it
	lis	%0111						; load our new bitmask
	com
pacman.draw.warpRightLoop:
	ds	0
	bm	pacman.draw.warpRightLoopEnd		; if we've reached the end of the 
	sl	1						; bits, end the loop.
	br	pacman.draw.warpRightLoop
pacman.draw.warpRightLoopEnd:
	lr	5, A						; save the bitmask
pacman.draw.warpEnd:

pacman.draw.sprite:					; draw the sprite
	SETISAR	PACMAN_X_REG
	; load x coordinate
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	; draw the sprite
	pi	sprite.draw

pacman.draw.end:
	pk							; return from the subroutine

;---------------------------------------------------------------------------
; Draw Pac-Man lives 
;---------------------------------------------------------------------------

pacman.drawLives:
	; save return address
	lr	K, P
	pi	kstack.push

	; erase the lives display
pacman.drawLives.erase:
 	; load sidebar color
	li	SIDEBAR_COLOR
	lr	1, A

	; start row loop
	li	56
	lr	3, A
pacman.drawLives.erase.row:
	li	97
	lr	2, A
pacman.drawLives.erase.column:
	pi	plot

pacman.drawLives.erase.checkLoop:
	; check column
	ds	2
	lr	A, 2
	ci	76						; see if we've reached the left edge
	bnz	pacman.drawLives.erase.column

	; check row
	ds	3
	lr	A, 3
	ci	51
	bnz	pacman.drawLives.erase.row			; branch if > 0

	; load the number of lives
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%00000111
	; draw a maximum of 4 lives
	ci	5
	bp	pacman.drawLives.setLivesCap
	lis	5
pacman.drawLives.setLivesCap:
	; use r10 as a loop counter
	lr	10, A
	
	; load the x coordinate
	li	78
	lr	11, A

pacman.drawLives.loop:
	; decrease the life counter
	ds	10
	; return if we're through
	bz	pacman.drawLives.end
	
	; color
	li	PACMAN_COLOR
	lr	1, A
	; x position
	lr	A, 11
	lr	2, A
	; y position
	li	52
	lr	3, A
	; sprite index
	lis	13
	lr	4, A
	; draw the life graphic
	pi	sprite.draw
	
	; increase the X coordinate
	lis	5
	as	11
	lr	11, A
	ci	94
	bp	pacman.drawLives.loop
	li	93
	lr	11,A
	br	pacman.drawLives.loop	

pacman.drawLives.end:
	; return from the subroutine
	pi	kstack.pop
	pk

;===========================================================================
; Ghost timer
;===========================================================================

set.slow.timer:

	; set ghost timer according to level
	
	; level 1-3, 6 and 10 normal "blue-time".		255	$FF
	; level 4: a little shorter				174	$ae (short)
	; level 5,7,8,11 and 14: a lot shorter		102	$66 (shorter)
	; level 9, 12, 13, 15, 16, 18	(seven dots) 	 36	$24 (shortest)
	; level 17, >18 no blue time				$3


	; save return address
	lr	K, P

	; save current global_timer_reg for attack/retreat waves
	SETISAR	GLOBAL_TIMER_REG
	ds	S
	bnz	set.slow.timer.backup.timer
	ds	S
set.slow.timer.backup.timer:
	lr	A, S
	SETISAR	ATTACK_TIMER_REG
	lr	S, A	

	SETISAR	LEVEL_NUMBER_REG
	lr	A, S


	; reads ROM table and sets timer
	; number between 0 and 18 sets slow timer

	dci	slow.ghost.timer.data			; sets DC to ROM table
	ci	18						; are we over level 18?
	bm	set.slow.timer.0				; yes - set timer to read at 0
	br	set.slow.timer.done			; else branch to adc
set.slow.timer.0:
	clr							; 0
set.slow.timer.done:
	adc							; Add A to DC
	lm							; read byte into A


	SETISAR	SLOW_TIMER_REG			; Set correct ISAR setting for register
	lr	S, A						; Store, from ROM, read byte in register

set.slow.timer.end:
	; return from the subroutine
	pk


;---------------------------------------------------------------------------
; Check ghost direction and mask off reverse direction
;---------------------------------------------------------------------------

valid.direction.and.mask:

	; save return adress
	lr	k, p
	pi	kstack.push

	; check that the ghosts's current direction is still valid
	SETISAR	GHOST_TMP_X_REG
	lr	A, I
	lr	4, A
	lr	A, I
	lr	5, A
	pi	checkIntersection
	ci	1
	bz	valid.direction.and.mask.inIntersection
	jmp	valid.direction.and.mask.end			; not in an intersection

valid.direction.and.mask.inIntersection:	
	; change direction
	; mask off reverse direction
	SETISAR	GHOST_TMP_DIR_REG
	lr	A, S
	ni	%0111
	bz	valid.direction.and.mask.mask.up
	ni	%0011
	bz	valid.direction.and.mask.mask.down
	ni	%0001
	bz	valid.direction.and.mask.mask.left
	; direction masking
valid.direction.and.mask.mask.right:
	lis	%1101
	br	valid.direction.and.mask.mask.end
valid.direction.and.mask.mask.left:
	lis	%1110
	br	valid.direction.and.mask.mask.end
valid.direction.and.mask.mask.down:
	lis	%0111
	br	valid.direction.and.mask.mask.end
valid.direction.and.mask.mask.up:
	lis	%1011
valid.direction.and.mask.mask.end:
	; save the masked direction
	ns	0
	lr	0, A
	; return from the subroutine
	pi	kstack.pop
	pk

valid.direction.and.mask.end:
	pi	kstack.pop
	li	$ff
	pk




;---------------------------------------------------------------------------
; Move ghost in and out of nest
;---------------------------------------------------------------------------


ghost.nest:

	; save return address
	lr	K, P
;	pi	kstack.push


	; clear the ghost
	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	SETISAR	GHOST_TMP_X_REG
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	; set sprite index
	clr
	lr	4, A
	; set bitmask
	lis	%0111
	com
	lr	5, A
	; draw the sprite
	pi	sprite.draw


	SETISAR	GHOST_TMP_DIR_REG
	lr	A, S
	ni	%01000000
	bnz	ghost.nest.out


	; increase y coordinate until ghost is inside nest
	; which is until y coordinate is 23

	li	%10000000					; set ghost to nested, no direction
	lr	D, A						; store new status
	lr	A, S						; load ghost_temp_y_reg
	ci	23						; check if we've reached 23
	bz	ghost.nest.in.image
	inc							; increase y-coordinate
	lr	D, A						; store in register
	li	39						; store x coordinate 39
	lr	S, A

	jmp	ghost.nest.end

	
ghost.nest.in.image:

	; non-eaten
	SETISAR	GHOST_NUMBER_REG
	lr	A, S
	lisl	4
	ci	3
	bz	ghost.nest.in.image.ghost3
	ci	2
	bz	ghost.nest.in.image.ghost2
	ci	1
	bz	ghost.nest.in.image.ghost1
 	; ghost 0 returned
 	lr	A, S
 	ni	%11101111
	lr	S, A
   	br	ghost.nest.in.image.changed
ghost.nest.in.image.ghost1:
 	lr	A, S
 	ni	%11011111
	lr	S, A
  	br	ghost.nest.in.image.changed
ghost.nest.in.image.ghost2:
 	lr	A, S
 	ni	%10111111
	lr	S, A
   	br	ghost.nest.in.image.changed	
ghost.nest.in.image.ghost3:
 	lr	A, S
 	ni	%01111111
	lr	S, A


ghost.nest.in.image.changed:

	SETISAR	GHOST_NUMBER_REG
	lr	A, S
	ci	2
	bz	ghost.nest.in.last.move.inky
	ci	1
	bz	ghost.nest.in.last.move.pokey
	br	ghost.nest.out

	; move to right side if it's Pokey (Ghost 1), x coordinate 48
ghost.nest.in.last.move.pokey:
	SETISAR	GHOST_TMP_X_REG
	lr	A, S
	ci	48
	bz	ghost.nest.out
	inc
	lr	S, A
	br	ghost.nest.end

	; move to left side if it's Inky (Ghost 2), x coordinate 30
ghost.nest.in.last.move.inky:
	SETISAR	GHOST_TMP_X_REG
	lr	A, S
	ci	30
	bz	ghost.nest.out
	ds	S
	br	ghost.nest.end

	; Blinky (Ghost 0) and Pinky (Ghost 3) turns in the middle
	br	ghost.nest.out

ghost.nest.out:
	SETISAR	GHOST_TMP_DIR_REG
	li	%01000000				; set register to nest out
	lr	S, A

	SETISAR	GHOST_TMP_X_REG		; check if ghost is in the middle
	lr	A, S
	ci	39
	bz	ghost.nest.out.others

	SETISAR	GHOST_NUMBER_REG		
	lr	A, S
	ci	2
	bz	ghost.nest.out.move.inky
	ci	1
	bz	ghost.nest.out.move.pokey
	br	ghost.nest.out.others

	; if it's Pokey (Ghost 1), move left to x-coordinate 39
ghost.nest.out.move.pokey:
	SETISAR	GHOST_TMP_X_REG
	ds	S
	br	ghost.nest.end

	; if it's Inky (Ghost 2), move right to x-coordinate 39
ghost.nest.out.move.inky:
	SETISAR	GHOST_TMP_X_REG
	lr	A, S
	inc
	lr	S, A
	br	ghost.nest.end

	; Ghost move up, y-coordinate decrease until it 
	; reaches 17, then ghost is no longer nested
ghost.nest.out.others
	SETISAR	GHOST_TMP_Y_REG
	lr	A, S
	ci	17
	bz	ghost.nest.ended
	ds	S
	br	ghost.nest.end

ghost.nest.ended:
	SETISAR	GHOST_TMP_DIR_REG
	lis	1						; set ghost to not nested, direction left
	lr	S, A						; store new status

ghost.nest.end:

	; redraw gate

	; draw the exit gate for the ghost
	li	GATE_COLOR
	lr	1,A
	li	39
	lr	2,A
	li	22
	lr	3,A
	li	19
	lr	4,A
	li	%11111000
	lr	5,A
	pi	sprite.draw	


	; return from the subroutine
;	pi	kstack.pop
	pk


;===========================================================================
; Copy player information 
;===========================================================================
;---------------------------------------------------------------------------
; Backup player 2
;---------------------------------------------------------------------------

; backup of player 2: ram + $B0


backup.players:

	; save return address
;	lr	K, P
;	pi	kstack.push

	; store a backup for player 2

	; load DC1 with the backup adress
	dci	[ram + $B0]
	xdc
	; load DC0 with the current ram
	dci	ram

	; start the copy cycle
	li	60
	lr	0, A
backup.player2.loop:
	; get and save a byte
	lm
	xdc
	st
	xdc

	; check if the loop has ended
	ds	0
	bnz	backup.player2.loop

	; store registers
	xdc
	SETISAR	HISCORE_REG
	lr	A, I					; store hiscore-registers (4 bytes)
	st
	lr	A, I
	st
	lr	A, I
	st
	lr	A, I
	st
	lr	A, I					; store score (4 bytes)
	st
	lr	A, I
	st
	lr	A, I
	st
	lr	A, S
	st
	SETISAR	LEVEL_NUMBER_REG
	lr	A, S	
	st						; store level
	SETISAR	PACMAN_LIVES_REG
	lr	A, I
	ni	%00000111
	st						; store lives
	lr	A, S
	ni	%00001111
	st						; store power pellets
	SETISAR	GLOBAL_TIMER_REG		
	lr	A, S
	st						; global timer reg
	SETISAR	BONUS_PRIZE_REG
	lr	A, S
	st						; store prize register

backup.players.end:
	; return from the subroutine
;	pi	kstack.pop
;	pk

	pop

;---------------------------------------------------------------------------
; Swap information 
;---------------------------------------------------------------------------


; copy current player information to backup

; backup of player 1: ram + $50
; backup of player 2: ram + $B0

swap.player:

	; save return address
;	lr	K, P
;	pi	kstack.push

	; load DC1 with the backup adress
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%01000000
	bnz	swap.player.two
	dci	[ram + $50]
	br	swap.player.adress.set
swap.player.two:
	dci	[ram + $B0]
swap.player.adress.set:
	xdc
	; load DC0 with the current ram
	dci	ram

	; start the copy cycle
	li	60
	lr	0, A
swap.player.loop:
	; get and save a byte
	lm
	xdc
	st
	xdc

	; check if the loop has ended
	ds	0
	bnz	swap.player.loop

	; store registers
	xdc
	SETISAR	HISCORE_REG
	lr	A, I					; store hi-score registers (4 bytes)
	st
	lr	A, I
	st
	lr	A, I
	st
	lr	A, I
	st
	lr	A, I					; store score (4 bytes)
	st
	lr	A, I
	st
	lr	A, I
	st
	lr	A, S
	st
	SETISAR	LEVEL_NUMBER_REG
	lr	A, S	
	st						; store level
	SETISAR	PACMAN_LIVES_REG
	lr	A, I
	ni	%00000111
	st						; store lives
	lr	A, S
	ni	%00001111
	st						; store power pellets
	SETISAR	GLOBAL_TIMER_REG		
	lr	A, S
	st
	SETISAR	BONUS_PRIZE_REG
	lr	A, S
	st						; store prize register
	


	; copy information from backup into original destination

	; load DC1 with ram
	dci	ram
	xdc
	; load DC0 with other backup
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%01000000
	bnz	swap.player.from.one
	dci	[ram + $B0]
	br	swap.player.adress.from.backup.set
swap.player.from.one:
	dci	[ram + $50]
swap.player.adress.from.backup.set:

	; start the copy cycle
	li	60
	lr	0, A
swap.player.from.backup.loop:
	; get and save a byte
	lm
	xdc
	st
	xdc

	; check if the loop has ended
	ds	0
	bnz	swap.player.from.backup.loop

	; fetch registers
	SETISAR	HISCORE_REG
	lm
	lr	I, A					; fetch hi-score-registers (4 bytes)
	lm
	lr	I, A
	lm
	lr	I, A
	lm
	lr	I, A
	lm
	lr	I, A					; fetch score (4 bytes)
	lm
	lr	I, A
	lm
	lr	I, A
	lm
	lr	S, A
	SETISAR	LEVEL_NUMBER_REG
	lm
	lr	S, A					; store level
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%11111000
	om
	lr	I, A
	lm
	lr	S, A					; store powerpellets
	SETISAR	GLOBAL_TIMER_REG
	lm
	lr	S, A
	SETISAR	BONUS_PRIZE_REG
	lm
	lr	S, A					; store prize register


	; swap player
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	ni	%01000000
	bnz	swap.player.register
	lr	A, S
	oi	%01000000
	lr	S, A
	br	swap.player.end
swap.player.register
	lr	A, S
	ni	%10111111
	lr	S, A

swap.player.end:
	; return from the subroutine
;	pi	kstack.pop
;	pk

	pop

	IF 	switch_music

;===========================================================================
; Music Functions 
;===========================================================================

;---------------------------------------------------------------------------
; Play Song 
;---------------------------------------------------------------------------
; plays a song from a memory location stored in DC0
; uses r3, r4, r5, r6, r7
; taken from Pro Football

playSong:                       				
	lis	3						; A = 3
	lr	4, A						; A -> r4, r4 = 3

.playSongDelay1:      
	inc							; increase A
	bnz	.playSongDelay1					; [Branch if not zero] back to .psdly1
	am							; Memory adressed by DC0 is added to A, flags set  - get duration data
	lr	7, A						; A -> r7
	bz	.playSongEnd					; Go to end if zero
	lm							; Load memory into A (adressed by DC0) - get frequency data
	lr	5, A						; A -> r5

	; check to see if we should delay
	inc
	bnc  .playSongLoop					; it didn't roll over, play a note instead

.playSongPause:
	li	$ff
	lr	6, A
.playSongPauseLoop:
	ds	6						; pause counter
	bnz	.playSongPauseLoop
	ds	7						; duration of the pause
	bnz	.playSongPause

	; play the next note
	br	playSong

.playSongLoop:
	lis	$8						; A= $80 
	sl	4
	outs	5						; Send  A -> port 5
	lr	A, 5						; r5 -> A

.playSongDelay2:
	inc							; increase A
	bnz	.playSongDelay2					; [Branch if not zero] back to .psdly2
	outs	5						; A -> port 5
	lr	A, 5						; r5 -> A

.playSongDelay3:
	inc							; increase A
	bnz	.playSongDelay3					; [Branch if not zero] back to .psdly3
	ds	4						; decrease r4
	bnz	.playSongLoop					; [Branch if not zero] back to .psloop
	lis	3						; A = 3
	lr	4, A						; A -> r4
	ds	7						; decrease r7
	bnz	.playSongLoop						; [Branch if not zero] back to .psloop
	br	playSong					; start over - branch to beginning

.playSongEnd:
	pop							; return from the subroutine


	ENDIF



;---------------------------------------------------------------------------
; intermission
;---------------------------------------------------------------------------


intermission:

	; clear screen to black, open middle grey area like in the intro

	; intro screen, b/w
	lis	$c
	sl	4
	lr	1, A

	pi	clearscreen.black	


	; set middle 25 pixels part to three color, grey bkg
	; change palette (rows [15,40])

	lis	1
	lr	1, A
	; horizontal position (122 = color row)
	li	122
	lr	2, A
	; vertical position
	li	27
	lr	3, A
	lr	4, A	
	lr	7, A
	pi	plot
intermission.grey.field.loop.on:
	ds	7
	lr	A, 7
	lr	3, A
	ci	14
	bz 	intermission.grey.field.on.end
	pi	plot
	
	; delay	
	lis	5
	lr	5, A
	pi	delay

	lr	A, 4
	inc
	lr	4, A
	lr	3, A
	pi	plot

	; delay	
	lis	5
	lr	5, A
	pi	delay

	br	intermission.grey.field.loop.on

intermission.grey.field.on.end:


	; read byte at $3FFE - if it's not $30 we're 
	; running MESS 0.129 or earlier or Pac-Man on Multi-Cart.

	dci	$3FFE
	lm	
	ci	$30
	bnz	intermission.delay.before.restart

	jmp	intermission.real
	; play music
	; run animation for this level
	; play music


intermission.delay.before.restart:


	; delay	
	li	$ff
	lr	5, A
	pi	delay


	; clear screen to black
	lis	$c
	sl	4
	lr	1, A
	pi	clearscreen.black	

	; set screen background back to grey
	pi	clearscreen


	; continue game

	jmp	level.init


;---------------------------------------------------------------------------
; Setup intersection data in RAM
;---------------------------------------------------------------------------


clearscreen:

	; set all pixels to background, set bkg on all rows.

	; start at 0,0

	clr
	lr	1, A
clearscreen.black:
	clr
	lr	2, A		; x coordinate
	lr	3, A		; y coordinate

	br	clr.plot
clr.next.x
	lr	2, A

clr.plot:
	; plot 
	lis	$C
	sl	4
	outs	1
	lr	A, 2
	com
	outs	4
	lr	A, 3
	com
	outs	5

	; transfer data to the screen memory
	lis	$6
	sl	4
	outs	0
	lis	$5
	sl	4
	outs	0


	; delay until it's fully updated
	lis	6
clr.plot.delay:	
	ai	$ff
	bnz	clr.plot.delay



	; add one to x coordinate
	lr	A, 2
	inc
	ci	111
	bnz	clr.next.x

clr.full.row:
	; change background color
	lr	A, 1
	outs	1
	li	126	
	com
	outs	4
	lr	A, 3
	com
	outs	5

	; transfer data to the screen memory
	lis	$6
	sl	4
	outs	0
	lis	$5
	sl	4
	outs	0

	; delay until it's fully updated
	lis	6
clr.plot.delay2:	
	ai	$ff
	bnz	clr.plot.delay2


	clr
	lr	2, A		; reset x-coordinate
	lr	A, 3		; increase y coordinate
	inc
	lr	3, A
	ci	64
	bnz	clr.plot


	pop



;===========================================================================
; Program Data 
;===========================================================================
;---------------------------------------------------------------------------
; Setup intersection data in RAM
;---------------------------------------------------------------------------
program.data.intersection:
	; store 1,0,0,0,0,0  11 times and finish with 1

	dci	ram + $200
	lis	11
	lr	1, A
	lis	1
program.data.intersection.loop:
	st							; 1
	clr
	st							; 0
	st							; 0
	st							; 0
	st							; 0
	st							; 0
	lis	1
	ds	1						; decrease loop counter
	bnz	program.data.intersection.loop
	st							; last 1
program.data.intersection.end:

	pop

;---------------------------------------------------------------------------

	; k stack functions
	include "kstack.inc"

	; graphic data
	include "graphics.inc"

	; data for the maze, the layout, and pellets
	include "maze.inc"

	; data table for Pokey, blue time, eatable, retreat coordinates, nest
	include "ROMtable.inc"

    IF switch_music
	; game music and sounds
	include "music.inc"
    ENDIF


	; five bytes left if not debug version


	IF	switch_debug_v

	ELSE

	org	[$2800 - 7 ]

; 7 bytes of data for Multi-Cart
himenu:
	lisu	3			; load the latch value in 'O'33, #r27
	lisl	3
	lr	A, S			
	st				; set the latch
	jmp	$0000			; jump to reset vector- BIOS will start normally

	ENDIF


	org	$2800
	
	.byte "$2800 This is where the 2kB RAM is placed - all the way until $3000...   "







; ----------------------------------------------------------
;    include intermission data
; ----------------------------------------------------------

	include "intermission.inc"


	; end


;===========================================================================
; Signature 
;===========================================================================

	org	[$3FFF - 4 - $10 -$10 ]

	.byte " b31 " 
	.byte " Blaoholtz  & "
	.byte	"Blackbird 2009"


